//******************************************************************************
// The software reads the accelaration on the axes X,Y,Z of the ADXL335 and
// writes the values to the LCD.
// The LCD must be connected as specified in the library included above.
//
// The ADXL335 Xout is connected to the ADC CH5 
// The ADXL335 Yout is connected to the ADC CH1 
// The ADXL335 Zout is connected to the ADC CH4 
// 
//
// Tested Platform: Breadboard
// Tested MCU: PIC32MX220F032B
// Tested Compiler: XC32 2.20
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h" 
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c" 

#include "module_ADC.h"
#include "module_ADC.c"

#include "LCD_ST7036.h"
#include "LCD_ST7036.c"

#define ADXL335_X_VALUE ADC_CH5
#define ADXL335_Y_VALUE ADC_CH1
#define ADXL335_Z_VALUE ADC_CH4
#define ADXL335_VCC 1024

#include "ADXL335.h"
#include "ADXL335.c"

void main(void) {
    int16_t x_acc = 0;
    int16_t y_acc = 0;
    int16_t z_acc = 0;
    
    IO_set_all_ports_as_inputs(); 
       
    LCD_initialize(20);
    LCD_backlight(LCD_TURN_ON_LED);    

    while (1){
       
        delay_ms (20);
        
        //Update component X
        x_acc = ADXL335_read_acceleration (ADXL335_X_VALUE);
        LCD_home();
        LCD_write_message("X: ");
        LCD_write_integer(x_acc,4, LCD_ZERO_CLEANING_ON);

        //Update component Y        
        y_acc = ADXL335_read_acceleration (ADXL335_Y_VALUE);        
        LCD_goto_xy(10,1);
        LCD_write_message("Y: ");
        LCD_write_integer(y_acc,4, LCD_ZERO_CLEANING_ON);
        
        //Update component Z       
        z_acc = ADXL335_read_acceleration (ADXL335_Z_VALUE);
        LCD_goto_line(2);
        LCD_write_message("Z: ");
        LCD_write_integer(z_acc,4, LCD_ZERO_CLEANING_ON);        

    }        
}
