//******************************************************************************
// The software reads the accelaration on the axes X,Y,Z of the ADX345 and
// writes the values to the LCD.
// The LCD must be connected as specified in the library included above.
// The ADX345 must be connected to the I2C Module 2 of the MCU.
//
//
// Tested Platform: Breadboard
// Tested MCU: PIC32MX220F032B
// Tested Compiler: XC32 2.20
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h" 
#include "module_I2C.c"

#include "LCD_ST7036.h"
#include "LCD_ST7036.c"

#define ADXL345_I2C_MODULE_2
#include "ADXL345.h"
#include "ADXL345.c"

void main(void) {
    
    int16_t x_acc = 0;
    int16_t y_acc = 0;
    int16_t z_acc = 0;
    
    IO_set_all_ports_as_inputs();
    
    ADXL345_initialize (400, ADXL345_RANGE_2g, ADXL345_SAMPLING_100);
    
    LCD_initialize(20);
    LCD_backlight(LCD_TURN_ON_LED);    
    
    while (1){
       
        delay_ms (20);
        
        //Update component X
        ADXL345_read_acceleration (ADXL345_X_VALUE, &x_acc);
        LCD_home();
        LCD_write_message("X: ");
        LCD_write_integer(x_acc,4, LCD_ZERO_CLEANING_ON);

        //Update component Y        
        ADXL345_read_acceleration (ADXL345_Y_VALUE, &y_acc);        
        LCD_goto_xy(10,1);
        LCD_write_message("Y: ");
        LCD_write_integer(y_acc,4, LCD_ZERO_CLEANING_ON);
        
         //Update component Z       
        ADXL345_read_acceleration (ADXL345_Z_VALUE, &z_acc);
        LCD_goto_line(2);
        LCD_write_message("Z: ");
        LCD_write_integer(z_acc,4, LCD_ZERO_CLEANING_ON);     
        
        ADXL345_sleep_mode (ADXL345_SLEEP_MODE_ON);
    }        
}
