#include <xc.h>

#include "LTlib.h"

#include "HW_mini_Sensing_MCU_Board.h"
#include "HW_mini_Sensing_Memory_Card.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_SPI.h"
#include "module_SPI.c"

#define MEMORY_SPI_MODULE_1
#include "memory_raw.h"
#include "memory_raw.c"

#include "FAT.h"
#include "FAT.c"

const uint8_t FILE_NAME[] = "Test33.txt";
const uint8_t TEXT_LENGTH = 26;

int main(void) {
    
    FATFS sd_card;	//Pointer to the mounted memory
    FIL file;		//File pointer

    uint16_t byte_written;

    uint8_t RX_buffer[TEXT_LENGTH];
    uint8_t Text_Demo[] = "Test: 1 2 3 4 5 6 7 8 9 10";
    uint16_t file_size = 0;
    uint16_t match_counter = 0;
    uint8_t index = 0;    
    
    IO_set_all_ports_as_inputs();
    
    //Enable pull up on MISO 
    //Depending on the connected board, and load on the pin, it helps the logic high level. 
    CNPUAbits.CNPUA8 = 0x01; 
    
    LED_1_DIR = SET_AS_OUTPUT;
    LED_2_DIR = SET_AS_OUTPUT;
    LED_1_MEMORY_DIR = SET_AS_OUTPUT;
   
    SPI1_open (SPI_MASTER_DEVICE);

    LED_1 = LED_ON; 
    delay_ms (200);

    LED_1 = LED_OFF; 
    delay_ms (200);
    
    blink_memory_LED (50);
    
    
    //*******************************************************
    // Write the file
    //*******************************************************
	if (f_mount(&sd_card, "", 1) == FR_OK) {	                                               
        
        if (f_open(&file, FILE_NAME, FA_CREATE_ALWAYS | FA_READ | FA_WRITE) == FR_OK) {	    
            f_write(&file, Text_Demo, TEXT_LENGTH, &byte_written);	                                        
            f_close(&file);
            
            // File properly written
            LED_2 = LED_ON;         
        } else {
            //Memory could be opened but file not written
            LED_1 = LED_ON;
        }
    } else {
        //Memory could not be mounted
        LED_1 = LED_ON; 
    }

    delay_ms (500);
    LED_2 = LED_OFF;  
    delay_ms (500);
    
    
    //*******************************************************
    // Read the file back
    //*******************************************************
    if (f_mount(&sd_card, "", 1) == FR_OK) {                                                    
        if (f_open(&file, FILE_NAME, FA_OPEN_EXISTING | FA_READ) == FR_OK) {                  
            
            if (f_size(&file)>0){                                                               
                file_size = f_size(&file);
                f_read(&file, RX_buffer,(f_size(&file)), &byte_written);                                
                f_close(&file);								                                    

                for (index = 0; index<TEXT_LENGTH; index++){
                    if (RX_buffer[index] == Text_Demo [index])
                        match_counter++;
                }
                 
                if (match_counter == TEXT_LENGTH){
                    LED_2 = LED_ON;
                } else {
                    //One or more bytes are different 
                    LED_1 = LED_ON;
                }                                                         
                  
            } else {
                //File is empty
                LED_1 = LED_ON;
            }
		} else {
            //File does not exist
            LED_1 = LED_ON; 
        }
	}
    
    while (1);
}
