//******************************************************************************
// The example writes and reads back a byte from an I2C
// EEPROM connected to the I2C bus Channel 2.
// If the readback is correct the LED connected to RA0
// is turned on, otherwise the LED connected to RA1
// 
// Connections
// The external A0,A1,A2 address pins are connected to ground
// LED1 connected to RA0 
// LED2 connected to RA1
//
//
// Tested Platform: Breadboard
// Tested MCU: PIC32MX220F032B
// Tested Compiler: XC32 2.20
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#define EEPROM_I2C_MODULE_2
#include "I2C_EEPROM.h"
#include "I2C_EEPROM.c"

#define LED_PORT IO_PORTA
#define LED_OK IO_BIT0
#define LED_FAIL IO_BIT1

void main(void) {
    
    uint8_t EEPROM_address = 0xA0;
    uint16_t memory_address = 0x0010;
    uint8_t write_data = 0xAA;
    uint8_t read_data = 0x00;
    
    IO_set_all_ports_as_inputs();
    
    IO_set_port_direction(LED_PORT, 0x00);
    
    I2C_EEPROM_initialize (100);
    
    I2C_EEPROM_write_byte (EEPROM_address, memory_address, write_data);
  
    //delay to make sure that the data is written before is read out
    delay_ms (10);
    
    I2C_EEPROM_read_byte (EEPROM_address,memory_address, &read_data);
    
    if (read_data == write_data){
        IO_write_port(LED_PORT, LED_OK);
        
    } else {
        IO_write_port(LED_PORT, LED_FAIL);
    }
   
    while(1);
}
