//******************************************************************************
// The software reads ID and Temperature out of the LSM6DSM sensor
// and writes it to the GLCD.
// The GLCD must be connected as specified in the library included above.
// The LSM6DSM must be connected to the I2C Module 1 of the MCU.  
// The LSM6DSM address bit is connected to GND (Device address: 0x6A)
//
//
// Tested Platform: mini Controller Board PIC32
// Tested MCU: PIC32MX150F128D
// Tested Compiler: XC32 4.30
//******************************************************************************

#include <xc.h>

#define SYSTEM_CLOCK 32000000UL
#include "LTlib.h"

#include "HW_mini_Sensing_PIC_Board.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "LSM6DSM.h"
#include "LSM6DSM.c"

#include "module_SPI.h"
#include "module_SPI.c"

#include "GLCD_ST7565.h"
#include "GLCD_ST7565.c"

#include "display_utilities.h"
#include "display_utilities.c"

#define DEVICE_ADDRESS 0x6A

void main (void) {
    
    uint8_t data = 0;
    uint8_t hex_string[6];
    
    int16_t temperature = 5;
    
    IO_set_all_ports_as_inputs();
    
    LED_2_DIR  = SET_AS_OUTPUT;
    
    delay_set_quartz(32);

    Display_initialize ();

    Display_backlight (DISPLAY_TURN_ON_LED);
    
    LSM6DSM_initialize (DEVICE_ADDRESS, 100);
    
    //Start the acc. to enable the temperature reading as well (on the same ADC)
    LSM6DSM_set_power_mode (DEVICE_ADDRESS, LSM6DSM_POWER_ON_ACC);
                   
    while(1){        
        
        delay_ms(500);
        
        LSM6DSM_get_device_ID (DEVICE_ADDRESS, &data); 
        LTlib_Hex8_to_string (hex_string, data);
        
        Display_write_message (5,5,"Device ID: 0x");
        Display_write_string (83,5, hex_string);
        
        Display_write_message (5,20,"Temperature:     C");
        LSM6DSM_get_temperature (DEVICE_ADDRESS, &temperature);
        Display_write_float (78,20,((float)temperature / 256), 4, 1, DISPLAY_ZERO_CLEANING_ON);
        
        if (data == LSM6DSM_DEVICE_ID){
            LED_2 ^= 0x01;
        } else {
            LED_2 = 0x01;
        }
          
    }
}

