//******************************************************************************
// The software reads the acceleration out of the LSM6DSM sensor
// and writes it to the GLCD.
// The GLCD must be connected as specified in the library included above.
// The LSM6DSM must be connected to the I2C Module 1 of the MCU.  
// The LSM6DSM address bit is connected to GND (Device address: 0x6A)
//
//
// Tested Platform: mini Controller Board PIC32
// Tested MCU: PIC32MX150F128D
// Tested Compiler: XC32 4.30
//******************************************************************************

#include <xc.h>

#define SYSTEM_CLOCK 32000000UL
#include "LTlib.h"

#include "HW_mini_Sensing_PIC_Board.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "LSM6DSM.h"
#include "LSM6DSM.c"

#include "module_SPI.h"
#include "module_SPI.c"

#include "GLCD_ST7565.h"
#include "GLCD_ST7565.c"

#include "display_utilities.h"
#include "display_utilities.c"

#define DEVICE_ADDRESS 0x6A

void main (void) {
    
    uint8_t data = 0;
    uint8_t hex_string[6];
    uint8_t status;
    
    float Acc_X = 0;
    float Acc_Y = 0;
    float Acc_Z = 0;
        
    IO_set_all_ports_as_inputs();
    
    LED_2_DIR  = SET_AS_OUTPUT;
    
    delay_set_quartz(32);
    
    Display_initialize ();
    Display_backlight (DISPLAY_TURN_ON_LED);
    
    LSM6DSM_initialize (DEVICE_ADDRESS, 400);
    LSM6DSM_set_full_scale (DEVICE_ADDRESS,LSM6DSM_ACCELEROMETER, LSM6DSM_ACC_FULLSCALE_8g);
    LSM6DSM_set_sampling_frequency (DEVICE_ADDRESS, LSM6DSM_ACCELEROMETER, LSM6DSM_ACC_SAMPLING_52_HZ);
    
    LSM6DSM_get_device_ID (DEVICE_ADDRESS, &data); 
    LTlib_Hex8_to_string (hex_string, data);

    Display_write_message (5,5,"Device ID: 0x");
    Display_write_string (83,5, hex_string);
    
    while(1){        
        
        LSM6DSM_get_interrupt_status (DEVICE_ADDRESS, &status);
        
        if (status & LSM6DSM_STATUS_XLDA){
            
            LSM6DSM_get_acceleration(DEVICE_ADDRESS, &Acc_X, &Acc_Y, &Acc_Z);
            Display_write_message (5,25,"Acc. X: ");

            Display_write_float (55,25,Acc_X, 6, 3, DISPLAY_ZERO_CLEANING_OFF);

            Display_write_message (5,35,"Acc. Y: ");
            Display_write_float (55,35,Acc_Y, 6, 3, DISPLAY_ZERO_CLEANING_OFF);

            Display_write_message (5,45,"Acc. Z: ");
            Display_write_float (55,45, Acc_Z, 6, 3, DISPLAY_ZERO_CLEANING_OFF);
        } else {
            Display_write_message (5,25,"No Acc. data ready...");
        }
        
        if (data == LSM6DSM_DEVICE_ID){
            LED_2 ^= 0x01;
        } else {
            LED_2 = 0x01;
        }
        
    }
}

