//******************************************************************************
// The software reads the lux out of the OPT3001. 
// The INT pin must be connected to a LED+R to Vcc. 
// It will be turned on once the light intesity rises the Lux high limit  
// and it will be turned off once it will fall below the low limit. 
//
//
// Tested Platform: mini Controller Board PIC32
// Tested MCU: PIC32MX150F128B
// Tested Compiler: XC32 4.30
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "LCD_ST7036.h"
#include "LCD_ST7036.c"

#define OPT3001_I2C_MODULE_2 
#include "OPT3001.h"
#include "OPT3001.c"

#define DEVICE_ADDRESS 0x44 //A0GND

void main (void){
    
    uint16_t data_readback = 0;
    float lux = 0;
    
    IO_set_all_ports_as_inputs ();
    
    LCD_initialize(20);
    
    OPT3001_initialize(400);
  
    OPT3001_configuration (DEVICE_ADDRESS, OTP3001_CONF_AUTO_FULL_SCALE | OTP3001_CONF_CONVERSION_TIME_800ms |
                           OTP3001_CONF_MODE_CONTINUOUS_SAMPLING | OTP3001_CONF_INT_PIN_AS_COMPARATOR |
                            OTP3001_CONF_INT_POLARITY_ACTIVE_LOW | OTP3001_CONF_FAULT_COUNTER_1);
    
    OPT3001_set_high_limit (DEVICE_ADDRESS, 1000);    
    OPT3001_set_low_limit (DEVICE_ADDRESS, 50);
      
    while (1){
        
        LCD_home ();
        LCD_write_message("Light: ");
        OPT3001_get_lux (DEVICE_ADDRESS, &lux);      
        LCD_write_integer((uint16_t) lux ,5, LCD_ZERO_CLEANING_ON);
        LCD_write_message(" lux");
        
        LCD_goto_line(3);

        LCD_write_message("Device: ");
        OPT3001_get_register (DEVICE_ADDRESS, OPT3001_REG_DEVICE_ID, &data_readback);
        
        if (data_readback == OPT3001_DEVICE_ID) {
            LCD_write_message("OPT3001");
        } else {
            LCD_write_message("Unkown");
        }

        OPT3001_get_register (DEVICE_ADDRESS, OPT3001_REG_CONFIGURATION, &data_readback); 
        delay_ms(500);
    }    
}
