//******************************************************************************
// The software is testing the basic functions implemented in the touch library
//
// Tested Platform: mini Controller Board PIC32
// Tested MCU: PIC32MX150F128D
// Tested Compiler: XC32 4.30
//******************************************************************************

#include <xc.h>

#define SYSTEM_CLOCK 32000000

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_ADC.h"
#include "module_ADC.c"

#include "module_SPI.h"
#include "module_SPI.c"

#include "GLCD_ST7565.h"
#include "GLCD_ST7565.c"

#include "display_utilities.h"
#include "display_utilities.c"

#include "Touch.h"
#include "Touch.c"

#define STATE_NONE 0
#define STATE_HOME 1
#define STATE_XY 2
#define STATE_DRAW 3

#define BUTTON_WIDTH 40
#define BUTTON_EIGTH 15

void draw_GUI (void);

//*************************************
//           
//*************************************
int main(void) {

    point touch;
    uint8_t state = STATE_HOME;
        
    IO_set_all_ports_as_inputs();
      
    delay_set_quartz(16);
   
    Display_initialize ();

    Display_backlight (DISPLAY_TURN_ON_LED);
    
    Display_set_font (DISPLAY_FONT_WIDTH_5, DISPLAY_FONT_HEIGTH_7);
        
    Display_write_message (20,28,"Calibration...");
    
    Touch_initialize (128, 64);
    
    Touch_beep (1);
    
    delay_ms (500);
    
    //*******************************
    //Calibration
    //*******************************

    Touch_calibrate ();    
       
    draw_GUI ();
        
    while (1) {    
        
        touch = Touch_measure(4);    

        
        //*********************************************
        //Check if a button got pressed
        //*********************************************
        
        //Button Clear
        if (((touch.X >2) && (touch.X <2+BUTTON_WIDTH)) && ((touch.Y >49) && (touch.Y <49+BUTTON_EIGTH))){
            
            Touch_beep(1);

            state = STATE_HOME;
        } 
        
        //Button XY
        if (((touch.X >44) && (touch.X <44+BUTTON_WIDTH)) && ((touch.Y >49) && (touch.Y <49+BUTTON_EIGTH))){ 
            
            Touch_beep(1);
            
            Display_draw_box(0,0,123,48,DISPLAY_FILLING_WHITE);  
            
            draw_GUI ();
            
            state = STATE_XY;
        } 
        
        //Button DRAW
        if (((touch.X >86) && (touch.X <86+BUTTON_WIDTH)) && ((touch.Y >49) && (touch.Y <49+BUTTON_EIGTH))){   
            Touch_beep(1);
            
            Display_draw_box(0,0,123,48,DISPLAY_FILLING_WHITE); 
            
            draw_GUI ();
            
            state = STATE_DRAW;
        } 
                    

        //*********************************************
        //Execute the activity according to the state
        //*********************************************
        
        if (state == STATE_HOME) {
            
            Display_draw_box(0,0,123,48,DISPLAY_FILLING_WHITE);      
            
            Display_write_message (20,20,"Select a mode.");
                                    
            draw_GUI ();
            
            state = STATE_NONE;
        }
        
        if (state == STATE_DRAW) {
              
            Display_plot_xy(touch.X, touch.Y, DISPLAY_FILLING_BLACK);  
        }
        
        if (state == STATE_XY) {
            Display_write_message (10,5,"X: ");
            Display_write_message (10,15,"Y: ");
            
            Display_write_integer(25,5, touch.X,3, DISPLAY_ZERO_CLEANING_OFF);
            Display_write_integer(25,15, touch.Y,3, DISPLAY_ZERO_CLEANING_OFF);            
        }

    }    
}

//*********************************
//         draw_GUI
//*********************************
void draw_GUI (void){
    
    Display_draw_button (2,49, BUTTON_WIDTH, BUTTON_EIGTH, "Home", DISPLAY_FILLING_BLACK);
    Display_draw_button (44,49, BUTTON_WIDTH, BUTTON_EIGTH, "XY", DISPLAY_FILLING_BLACK);
    Display_draw_button (86,49, BUTTON_WIDTH, BUTTON_EIGTH, "Draw", DISPLAY_FILLING_BLACK);
}
    