//******************************************************************************
// The example reads continuously the ADC channels CH1 and CH5 of the MCU.
// A LM35 sensor is connected to CH5.
// A light sensor is connected to CH1.
// The data read from the ADC channels are written to the LCD. 
// The LCD must be connected reflecting the settings of the LCD included library.
//
// Tested Platform: Breadboard
// Tested MCU: PIC32MX220F032
// Tested Compiler: XC32 2.20
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_ADC.h"
#include "module_ADC.c"

#include "LCD_ST7036.h"
#include "LCD_ST7036.c"

//Temperature channel
#define TEMPERATURE_CH ADC_CH5

//Light channel
#define LIGHT_CH ADC_CH1

void main (void){
    
    uint16_t light_value = 0;
    uint16_t temperature_value = 0;
    
    IO_set_all_ports_as_inputs();
    
    LCD_initialize(20);
    
    ADC_open(LIGHT_CH + TEMPERATURE_CH);
    
    while(1){
        
        LCD_home();
        
        ADC_select_channel (TEMPERATURE_CH);
        ADC_start_conversion();
        while(ADC_is_converting());
        temperature_value = ADC_read_value()/2;
        
        ADC_select_channel (LIGHT_CH);
        ADC_start_conversion();
        while(ADC_is_converting());
        light_value = ADC_read_value();
        
        LCD_write_message("Temp  : ");
        LCD_write_integer (temperature_value, 4, LCD_ZERO_CLEANING_ON); 
        LCD_write_char (223);
        LCD_write_char ('C');
        
        LCD_goto_line(2);
        
        LCD_write_message("Light : ");
        LCD_write_integer (light_value, 4, LCD_ZERO_CLEANING_ON);          
    }
}
