//******************************************************************************
// The software erase the flash memory block 0x2000. Afterward it writes the
// buffer with dummy data (0,1,2,3,4...) and writes it back to the flash.
// After the write process all the data is read back and written byte by byte
// to the PORTD where the 8x LED are connected. 
//
// Tested Platform: Breadboard
// Tested MCU: PIC32MX220F032
// Tested Compiler: XC32 2.20
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "module_FLASH.h"
#include "module_FLASH.c"

#include "module_IO.h"
#include "module_IO.c"

void main(void) {
    
    uint32_t data_buffer[FLASH_WRITE_BLOCK];
    uint8_t index = 0;
    
    uint32_t base_address = 0x9d000000;
    uint32_t offset_address = 0x3000;
    uint32_t address = base_address + offset_address;
    
    IO_set_all_ports_as_inputs();
    
    //Erase the memory block
    //FLASH_erase_memory(0x2000);
    //FLASH_WRITE_BLOCK
    
    //Clean the buffer
    for (index=0; index < 5; index++ ){
        data_buffer[index] = 0x00;
    }
    
    FLASH_read_memory(address,5, &data_buffer[0]);
       
    
    //Fill the buffer with the index value.
    for (index=0; index < 5; index++ ){
        data_buffer[index] = index;
    }
    
    //This function uses phisical address, thus the conversion with the & 0x1FFFFFFF    
    FLASH_write_memory(address & 0x1FFFFFFF, FLASH_WRITE_BLOCK, &data_buffer[0]);
  
 
    //Clean the buffer
    for (index=0; index < 5; index++ ){
        data_buffer[index] = 0x00;
    }
    
    FLASH_read_memory(address,5, &data_buffer[0]);
       
    while (1);
}
