//******************************************************************************
// In this example the nRF24L01 is used to TX and RX some data out of two buttons.
// Pressing BT1 one LED is turned ON, pressing BT2 another LED is turned ON.
//
//
// Tested Platform: mini Sensing Board
// Tested MCU: PIC32MX220F128D
// Tested Compiler: XC32 2.50
//******************************************************************************

#include <xc.h>

#define SYSTEM_CLOCK 32000000UL

#include "LTlib.h"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "HW_mini_Sensing_PIC_Board.h"
#include "HW_mini_Sensing_USB.h"

#include "module_IO.h"
#include "module_IO.c"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "module_SPI.h"
#include "module_SPI.c"

#define NRF24L01_SPI_MODULE_1 
#include "nRF24L01.h"
#include "nRF24L01.c"

#define ADDRESS_SIZE NRF24L01_ADDRESS_5_BYTES
#define PAYLOAD_SIZE 20
#define TX_CHANNEL 10

void main (void) {
    
    NRF24L01_packet packet;
    
    uint8_t address[NRF24L01_ADDRESS_MAX_SIZE];

    IO_set_all_ports_as_inputs();
    
    LED_1_DIR = SET_AS_OUTPUT;
    LED_2_DIR = SET_AS_OUTPUT;
    
    SW_1_USB_DIR = SET_AS_INPUT;
    SW_2_USB_DIR = SET_AS_INPUT;
    
    IO_enable_pull_up_resistors(IO_PORTB, IO_BIT4 + IO_BIT7);
    
    LED_1 = ON_STATE;
    LED_2 = OFF_STATE;

    NRF24L01_initialize (TX_CHANNEL, NRF24L01_RF_POWER_0dBm, NRF24L01_BIT_RATE_1MHZ);
       
    NRF24L01_packet_format (NRF24L01_ADDRESS_5_BYTES, NRF24L01_CRC_ENABLED, NRF24L01_CRC_SIZE_1_BYTE);
       
    address[0] = 0x01;
    address[1] = 0x02;
    address[2] = 0x03;  
    address[3] = 0x04; 
    address[4] = 0x05;    
    NRF24L01_set_address (NRF24L01_ADDRESS_TX, address, ADDRESS_SIZE);
    
    //Set Pipe P0 address as the TX address
    NRF24L01_set_address (NRF24L01_ADDRESS_RX_P0, address, ADDRESS_SIZE);
    NRF24L01_set_pipe (NRF24L01_P0_SIZE, PAYLOAD_SIZE);
    NRF24L01_pipe_configuration (NRF24L01_PIPE_0_EN, NRF24L01_PIPE_ENABLED);  
     
    LED_1 = OFF_STATE;

    while (1){
        
        packet.size = PAYLOAD_SIZE;
       
        //**********************************************************************
        //Check if any data has been received
        //**********************************************************************        
        //Turn on the RX mode
        NRF24L01_set_RX_mode (NRF24L01_RX_MODE_ON);
        delay_ms (1);
        
        
        if (NRF24L01_is_data_received() != NRF24L01_PIPES_EMPTY){
                        
            packet.size = PAYLOAD_SIZE;        
            NRF24L01_read_data (&packet);
                        
            if (packet.payload[0] == 0x01){
                LED_1 = ON_STATE; 
            }
            
            if (packet.payload[0] == 0x02){
                LED_2 = ON_STATE; 
            }
            
            //The LED are kept ON for few ms
            delay_ms (5);
            
        } else {
            
            LED_1 = OFF_STATE;  
            LED_2 = OFF_STATE;     
        }
        
        
        //**********************************************************************
        //Check if any button has been pressed
        //**********************************************************************
        if (SW_1_USB == BUTTON_PRESSED){
            delay_ms (10);
            
            if (SW_1_USB == BUTTON_PRESSED){
                
                packet.payload[0] = 0x01;
                
                NRF24L01_send_data (packet);
                
                LED_1 ^= ON_STATE;
                delay_ms (5);
                LED_1 ^= ON_STATE;  
            }            
        }
        
        
        
        if (SW_2_USB == BUTTON_PRESSED){
            delay_ms (10);
            
            if (SW_2_USB == BUTTON_PRESSED){
                
                packet.payload[0] = 0x02;
                
                NRF24L01_send_data (packet);
                
                LED_1 ^= ON_STATE;
                delay_ms (5);
                LED_1 ^= ON_STATE;  
            }            
        }   
    
    }
}


