//******************************************************************************
// In this example the DAC MCP4822 is set to generate a voltage reference.
// The MCP4822 is connected to the SPI module 1 but it can be connected also 
// to module 2 by changing the definition before the library. 
// Check the device configuration file for the peripheral pin out.
//
// The voltage is set as fixed one to 1.8V but different ones can be selected.
//
//
// Tested Platform: Breadboard
// Tested MCU: PIC32MX220F032B
// Tested Compiler: XC32 2.20
//******************************************************************************

#include <xc.h>

#define SYSTEM_CLOCK 32000000UL

#include "LTlib.h"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "HW_mini_Sensing_PIC_Board.h"

#include "module_IO.h"
#include "module_IO.c"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "module_SPI.h"
#include "module_SPI.c"

#include "module_UART.h"  
#include "module_UART.c" 

#define NRF24L01_SPI_MODULE_1 
#include "nRF24L01.h"
#include "nRF24L01.c"

#define ADDRESS_SIZE NRF24L01_ADDRESS_5_BYTES
#define PAYLOAD_SIZE 20
#define TX_CHANNEL 10

void main (void) {
    
    NRF24L01_packet packet;
    uint8_t counter =0;
    uint16_t refresh_counter =0;
    
    uint8_t address[NRF24L01_ADDRESS_MAX_SIZE];
    uint8_t register_data = 0;
    uint8_t register_hex [6];

    IO_set_all_ports_as_inputs();
    
    LED_1_DIR = SET_AS_OUTPUT;
    LED_2_DIR = SET_AS_OUTPUT;
    
    LED_1 = ON_STATE;
    LED_2 = OFF_STATE;

    UART1_open (UART_BAUDRATE_115200); 
    UART1_write_message ("Booting..."); 
    UART1_write_byte(10);
    
    NRF24L01_initialize (TX_CHANNEL, NRF24L01_RF_POWER_0dBm, NRF24L01_BIT_RATE_1MHZ);
 
    address[0] = 0x01;
    address[1] = 0x02;
    address[2] = 0x03;  
    address[3] = 0x04; 
    address[4] = 0x05; 
    
    NRF24L01_set_address (NRF24L01_ADDRESS_TX, address, ADDRESS_SIZE);
    
    //Set Pipe P0 address as the TX address
    NRF24L01_set_address (NRF24L01_ADDRESS_RX_P0, address, ADDRESS_SIZE);
    NRF24L01_set_pipe (NRF24L01_P0_SIZE, PAYLOAD_SIZE);
    NRF24L01_pipe_configuration (NRF24L01_PIPE_0_EN, NRF24L01_PIPE_ENABLED); 
     
    LED_1 = OFF_STATE;

    //Turn on the RX mode
    NRF24L01_set_RX_mode (NRF24L01_RX_MODE_ON);
        
    delay_ms (1);
    
    while (1){
        
        //delay_ms (2);
             
        
//        packet.size = PAYLOAD_SIZE;
//        
//        for (counter = 0; counter < PAYLOAD_SIZE; counter++){
//            packet.payload[counter] = counter;
//        }
//        
//        NRF24L01_send_data (packet);
        
//        NRF24L01_set_power_mode (NRF24L01_TURN_ON);
//        NRF24L01_update_register_bit (NRF24L01_REG_CONFIG, NRF24L01_CONFIG_BIT_PWR_UP, 1);
//        NRF24L01_update_register_bit (NRF24L01_REG_CONFIG, NRF24L01_CONFIG_BIT_PRIM_RX, 1);
        
//        UART1_write_byte(8);
//        
//        refresh_counter++;
//        LTlib_int16_to_string (register_hex, refresh_counter);
//        UART1_write_string("Refresh Counter: ");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_STATUS);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("Reg. Status: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_CONFIG);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("Reg. Config: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_OBSERVE_TX);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("Reg. Observe TX: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_EN_RXADDR);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("Reg. Enable RX Pipe: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_FIFO_STATUS);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("Reg. FIFO Status: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_CD);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("Reg. Carrier Detected: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_EN_AA);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("Reg. EN_AA: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_SETUP_AW);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("Reg. SETUP_AW: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_SETUP_RETR);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("Reg. SETUP_RETR: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_RF_CH);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("RF CH: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_RF_SETUP);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("RF Setup: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_RX_PW_P0);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("RX PW P0: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_DYNPD);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("Reg. DYNPD: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        register_data = NRF24L01_get_register (NRF24L01_REG_FEATURE);        
//        LTlib_Hex8_to_string (register_hex, register_data);
//        UART1_write_string("Reg. Feature: 0x");
//        UART1_write_string(register_hex);
//        UART1_write_byte(10);
//        
//        
//        address = NRF24L01_get_address (NRF24L01_REG_TX_ADDR, NRF24L01_ADDRESS_MAX_SIZE);
//        UART1_write_string("Reg. TX Address    : ");
//        
//        for (counter = 0; counter < NRF24L01_ADDRESS_MAX_SIZE; counter++){
//            LTlib_Hex8_to_string (register_hex, address[counter]);
//            UART1_write_string(" 0x");
//            UART1_write_string(register_hex);        
//        }       
//        
//        UART1_write_byte(10);
//        
//        address = NRF24L01_get_address (NRF24L01_REG_RX_ADDR_P0, NRF24L01_ADDRESS_MAX_SIZE);
//        UART1_write_string("Reg. RX P0 Address : ");
//        
//        for (counter = 0; counter < NRF24L01_ADDRESS_MAX_SIZE; counter++){
//            LTlib_Hex8_to_string (register_hex, address[counter]);
//            UART1_write_string(" 0x");
//            UART1_write_string(register_hex);        
//        }       
//        
//        UART1_write_byte(10);
//        
//        
//        address = NRF24L01_get_address (NRF24L01_REG_RX_ADDR_P1, NRF24L01_ADDRESS_MAX_SIZE);
//        UART1_write_string("Reg. RX P1 Address : ");
//        
//        for (counter = 0; counter < NRF24L01_ADDRESS_MAX_SIZE; counter++){
//            LTlib_Hex8_to_string (register_hex, address[counter]);
//            UART1_write_string(" 0x");
//            UART1_write_string(register_hex);        
//        }       
//        
//        UART1_write_byte(10);
        
        
        
        if (NRF24L01_is_data_received() != NRF24L01_PIPES_EMPTY){
                        
            packet.size = PAYLOAD_SIZE;
        
            NRF24L01_read_data (&packet);
            
            if (packet.payload[0] == 0x01){
                LED_1 = ON_STATE; 
            }
            
            if (packet.payload[0] == 0x02){
                LED_2 = ON_STATE; 
            }
            

            UART1_write_string("Received data : ");

            for (counter = 0; counter < PAYLOAD_SIZE; counter++){
                LTlib_Hex8_to_string (register_hex, packet.payload[counter]);
                UART1_write_string(" 0x");
                UART1_write_string(register_hex);        
            }       

            UART1_write_byte(10);

            
        } else {
            
            LED_1 = OFF_STATE;  
            LED_2 = OFF_STATE;  
            
            UART1_write_string("Received data : ");

            for (counter = 0; counter < PAYLOAD_SIZE; counter++){
                UART1_write_string(" 0x00");
            }      
        }
        
    }
}


