/*******************************************************************************
 
Autore : Mauro Laurenti
Versione : 1.0
 
Created on Date : 10/11/2022
Last update     : 10/11/2022

CopyRight 2006-2022 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "module_IO.h"


//************************************************************
//      IO_set_all_port_as_inputs function implementation
//************************************************************
void IO_set_all_ports_as_inputs (void){
                
    #if (NUMBER_OF_IO_PORTS == 2)

        LATA = 0x0000;
        LATB = 0x0000;

        TRISA = 0xFFFF;        
		TRISB = 0xFFFF;
        
        ODCA = 0x0000;
        ODCB = 0x0000;

        //Disable ADC buffer        
            
        #ifdef ADC_WITH_ANSEL_REG
            ADC_PORT_CONFIGURATION_1 = 0x0000;
            ADC_PORT_CONFIGURATION_2 = 0x0000;
        #endif   
            
        #ifdef ADC_WITH_ADxPCFG_REG
            ADC_PORT_CONFIGURATION_1 = 0xFFFF;
        #endif 
            
	#endif

    #if (NUMBER_OF_IO_PORTS == 3)

        LATA = 0x0000;
        LATB = 0x0000;
        LATC = 0x0000;

        TRISA = 0xFFFF;        
		TRISB = 0xFFFF;
        TRISC = 0xFFFF;
        
        ODCA = 0x0000;
        ODCB = 0x0000;
        ODCC = 0x0000;

        //Disable ADC buffer        
            
        #ifdef ADC_WITH_ANSEL_REG
            ADC_PORT_CONFIGURATION_1 = 0x0000;
            ADC_PORT_CONFIGURATION_2 = 0x0000;
            ADC_PORT_CONFIGURATION_3 = 0x0000;
        #endif  
            
        #ifdef ADC_WITH_ADxPCFG_REG
            ADC_PORT_CONFIGURATION_1 = 0xFFFF;
        #endif   
            
	#endif        
	 	
}


//************************************************************
//      IO_set_port_direction function implementation
//************************************************************
void IO_set_port_direction (uint8_t mcu_port, port_size port_direction){
       
    #if (NUMBER_OF_IO_PORTS == 2)

        if (mcu_port == IO_PORTA){
            TRISA = port_direction;
        }

        if (mcu_port == IO_PORTB){
            TRISB = port_direction;
        }                
    #endif  
        
    #if (NUMBER_OF_IO_PORTS == 3)

        if (mcu_port == IO_PORTA){
            TRISA = port_direction;
        }

        if (mcu_port == IO_PORTB){
            TRISB = port_direction;
        }    
        
        if (mcu_port == IO_PORTC){
            TRISC = port_direction;
        }    
    #endif      
        
}


//************************************************************
//      IO_set_port_direction function implementation
//************************************************************
void IO_write_port (uint8_t mcu_port, port_size port_data){
    

    #if (NUMBER_OF_IO_PORTS == 2)

        if (mcu_port == IO_PORTA){
            LATA = port_data;
        }

        if (mcu_port == IO_PORTB){
            LATB = port_data;
        }
            
	#endif   

    
    #if (NUMBER_OF_IO_PORTS == 3)

        if (mcu_port == IO_PORTA){
            LATA = port_data;
        }

        if (mcu_port == IO_PORTB){
            LATB = port_data;
        }
        
        if (mcu_port == IO_PORTC){
            LATC = port_data;
        }
            
	#endif      
        
            
}


//************************************************************
//      IO_read_port function implementation
//************************************************************
port_size IO_read_port (uint8_t mcu_port){
    
        
    #if (NUMBER_OF_IO_PORTS == 2)

        if (mcu_port == IO_PORTA){
            return (PORTA);
        }

        if (mcu_port == IO_PORTB){
            return (PORTB);
        }
        
    #endif  
        
            
    #if (NUMBER_OF_IO_PORTS == 3)

        if (mcu_port == IO_PORTA){
            return (PORTA);
        }

        if (mcu_port == IO_PORTB){
            return (PORTB);
        }
        
        if (mcu_port == IO_PORTC){
            return (PORTC);
        }
        
    #endif     
        
    
   return (0);     
}


//************************************************************
//      IO_read_port_bit function implementation
//************************************************************
port_size IO_read_port_bit (uint8_t mcu_port, port_size bit_mask ){

    port_size port_data = 0;

    #if (NUMBER_OF_IO_PORTS == 2)

        if (mcu_port == IO_PORTA){
            port_data = PORTA;
        }

        if (mcu_port == IO_PORTB){
            port_data = PORTB;
        }

        if ( (port_data & bit_mask) == 0){
            return (0);
        } else {
            return (1);
        }

    #endif 

    #if (NUMBER_OF_IO_PORTS == 3)

        if (mcu_port == IO_PORTA){
            port_data = PORTA;
        }

        if (mcu_port == IO_PORTB){
            port_data = PORTB;
        }
    
        if (mcu_port == IO_PORTC){
            port_data = PORTC;
        }

        if ( (port_data & bit_mask) == 0){
            return (0);
        } else {
            return (1);
        }

    #endif 
    
  
}

//************************************************************
//      IO_enable_pull_up_resitors function implementation
//************************************************************
#ifdef PULL_UP_RESISTORS_AVAILABLE
void IO_enable_pull_up_resistors (uint8_t mcu_port, port_size resistors_to_enable) {
    
    #if (NUMBER_OF_IO_PORTS == 2)
        if (mcu_port == IO_PORTA){
            PULL_UP_ENABLE_REGISTER_A = resistors_to_enable;
        }

        if (mcu_port == IO_PORTB){
            PULL_UP_ENABLE_REGISTER_B = resistors_to_enable;
        }      
    #endif

    #if (NUMBER_OF_IO_PORTS == 3)
        if (mcu_port == IO_PORTA){
            PULL_UP_ENABLE_REGISTER_A = resistors_to_enable;
        }

        if (mcu_port == IO_PORTB){
            PULL_UP_ENABLE_REGISTER_B = resistors_to_enable;
        }  
        
        if (mcu_port == IO_PORTC){
            PULL_UP_ENABLE_REGISTER_C = resistors_to_enable;
        } 
        
    #endif
    
}
#endif

//************************************************************
//      IO_enable_pull_down_resistors function implementation
//************************************************************
#ifdef PULL_DOWN_RESISTORS_AVAILABLE
void IO_enable_pull_down_resistors (uint8_t mcu_port, port_size resistors_to_enable){
    
    #if (NUMBER_OF_IO_PORTS == 2)
        if (mcu_port == IO_PORTA){
            PULL_DOWN_ENABLE_REGISTER_A = resistors_to_enable;
        }

        if (mcu_port == IO_PORTB){
            PULL_DOWN_ENABLE_REGISTER_B = resistors_to_enable;
        }      
    #endif 
        
    #if (NUMBER_OF_IO_PORTS == 3)
        if (mcu_port == IO_PORTA){
            PULL_DOWN_ENABLE_REGISTER_A = resistors_to_enable;
        }

        if (mcu_port == IO_PORTB){
            PULL_DOWN_ENABLE_REGISTER_B = resistors_to_enable;
        } 
        
        if (mcu_port == IO_PORTC){
            PULL_DOWN_ENABLE_REGISTER_C = resistors_to_enable;
        }        
    #endif      
}

#endif