/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 24/02/2023
Last update     : 24/02/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library contains all the functions that are handy for controlling 			
the SPI module
*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "module_SPI.h"


//************************************************************
//               SPI_open  Implementation
//************************************************************
void SPI1_open (uint8_t device_type){
    
    uint16_t data = 0x00; 
    
    #ifndef MCC_USED
        // unlock PPS
        __builtin_write_OSCCONL(OSCCON & 0xbf); 
        //Pin mux initialization
        SPI1_SCK_LINE;			         
        SPI1_SDI_LINE;			 
        SPI1_SDO_LINE;		
        SPI1_SS_LINE;	
        // lock PPS
        __builtin_write_OSCCONL(OSCCON | 0x40); 
    #endif

    //Disable the SPI module
    SPI1_ENABLE = 0x00;
    
    if (device_type == SPI_MASTER_DEVICE){
        SPI1_CONFIG_REG1.MSTEN = 0x01;
    } else {
        SPI1_CONFIG_REG1.MSTEN = 0x00;
    }
        
    //Dummy read to clear the buffer
    data = SPI1_BUFFER;
    
    // Clear Interrupts flags
    SPI1_IF = 0x00;
    SPI1_TXIF = 0x00;
    SPI1_RXIF = 0x00;
    
    //Reset errors : Frame, Overflow, Underflow
    SPI1_FRMERR = 0x00;
    SPI1_SPIROV = 0x00;
    SPI1_SPITUR = 0x00;    
    
    //Enable SPI module, ready for RX/TX
    SPI1_ENABLE = 0x01;
}

#ifdef SPI2_AVAILABLE
    void SPI2_open (uint8_t device_type){
        uint16_t data; 
    
        #ifndef MCC_USED
            // unlock PPS
            __builtin_write_OSCCONL(OSCCON & 0xbf); 
            //Pin mux initialization
            SPI2_SCK_LINE;			         
            SPI2_SDI_LINE;			 
            SPI2_SDO_LINE;		
            SPI2_SS_LINE;	
            // lock PPS
            __builtin_write_OSCCONL(OSCCON | 0x40); 
        #endif

        //Disable the SPI module
        SPI2_ENABLE = 0x00;

        if (device_type == SPI_MASTER_DEVICE){
            SPI2_CONFIG_REG1.MSTEN = 0x01;
        } else {
            SPI2_CONFIG_REG1.MSTEN = 0x00;
        }

        //Dummy read to clear the buffer
        data = SPI2_BUFFER;

        // Clear Interrupts flags
        SPI2_IF = 0x00;
        SPI2_TXIF = 0x00;
        SPI2_RXIF = 0x00; 

        //Reset errors : Frame, Overflow, Underflow
        SPI2_FRMERR = 0x00;
        SPI2_SPIROV = 0x00;
        SPI2_SPITUR = 0x00;    

        //Enable SPI module, ready for RX/TX
        SPI2_ENABLE = 0x01;
    }
#endif

#ifdef SPI3_AVAILABLE
    void SPI3_open (uint8_t device_type){
        uint16_t data; 
    
        #ifndef MCC_USED
            // unlock PPS
            __builtin_write_OSCCONL(OSCCON & 0xbf); 
            //Pin mux initialization
            SPI3_SCK_LINE;			         
            SPI3_SDI_LINE;			 
            SPI3_SDO_LINE;		
            SPI3_SS_LINE;	
            // lock PPS
            __builtin_write_OSCCONL(OSCCON | 0x40); 
        #endif

        //Disable the SPI module
        SPI3CON1Lbits.SPIEN = 0x00;

        if (device_type == SPI_MASTER_DEVICE){
            SPI3CON1Lbits.MSTEN = 0x01;
        } else {
            SPI3CON1Lbits.MSTEN = 0x00;
        }

        //Dummy read to clear the buffer
        data = SPI3_BUFFER;

        // Clear Interrupts flags
        IFS5bits.SPI3IF = 0x00;
        IFS5bits.SPI3TXIF = 0x00;
        IFS3bits.SPI3RXIF = 0x00; 

        //Reset errors : Frame, Overflow, Underflow
        SPI3STATLbits.FRMERR = 0x00;
        SPI3STATLbits.SPIROV = 0x00;
        SPI3STATLbits.SPITUR = 0x00;    

        //Enable SPI module, ready for RX/TX
        SPI3CON1Lbits.SPIEN = 0x01;
    }
#endif    
    
    

//************************************************************
//               SPI1_close  Implementation
//************************************************************
void SPI1_close (void){
    SPI1_ENABLE = 0x00;
}

#ifdef SPI2_AVAILABLE
    void SPI2_close (void){
        SPI2_ENABLE = 0x00;
    }
#endif
    
#ifdef SPI3_AVAILABLE
    void SPI3_close (void){
        SPI3_CONFIG_REG1.SPIEN = 0x00;
    }
#endif    

//************************************************************
//               SPI1_baudrate  Implementation
//************************************************************
void SPI1_baudrate (uint8_t baudrate){

    #ifdef SPI_BAUDRATE_BRG
        SPI1_BAUDRATE_REG = baudrate;
    #endif

    #ifdef SPI_BAUDRATE_PRI_SEC_PRESCALER
        //Set to 0 the prescaler bits
        SPI1_BAUDRATE_REG &=  0xFFE0;
        
        //Set the prescaler bits
        SPI1_BAUDRATE_REG |= baudrate;
    #endif
}

#ifdef SPI2_AVAILABLE
    void SPI2_baudrate (uint8_t baudrate){
        #ifdef SPI_BAUDRATE_BRG
            SPI2_BAUDRATE_REG = baudrate;
        #endif

        #ifdef SPI_BAUDRATE_PRI_SEC_PRESCALER
            //Set to 0 the prescaler bits
            SPI2_BAUDRATE_REG &=  0xFFE0;

            //Set the prescaler bits
            SPI2_BAUDRATE_REG |= baudrate;
        #endif
    }
#endif
    
#ifdef SPI3_AVAILABLE
    void SPI3_baudrate (uint8_t baudrate){
        #ifdef SPI_BAUDRATE_BRG
            SPI3_BAUDRATE_REG = baudrate;
        #endif

        #ifdef SPI_BAUDRATE_PRI_SEC_PRESCALER
            //Set to 0 the prescaler bits
            SPI3_BAUDRATE_REG &=  0xFFE0;

            //Set the prescaler bits
            SPI3_BAUDRATE_REG |= baudrate;
        #endif
    }
#endif    

//************************************************************
//               SPI1_mode  Implementation
//************************************************************
void SPI1_mode (uint8_t communication_mode){
    
    switch (communication_mode){
        case SPI_MODE_0: // Data is sent on the rising edge
                        SPI1_CONFIG_REG1.CKE = 0x01;  
                        SPI1_CONFIG_REG1.CKP = 0x00;
                        break;
                        
        case SPI_MODE_1: 
                        SPI1_CONFIG_REG1.CKE = 0x00;  
                        SPI1_CONFIG_REG1.CKP = 0x00;      
                        break;
                        
        case SPI_MODE_2: 
                        SPI1_CONFIG_REG1.CKE = 0x01;  
                        SPI1_CONFIG_REG1.CKP = 0x01;        
                        break;
                        
        case SPI_MODE_3:
                        SPI1_CONFIG_REG1.CKE = 0x00;  
                        SPI1_CONFIG_REG1.CKP = 0x01; 
                        break;                
    }            
}

#ifdef SPI2_AVAILABLE
    void SPI2_mode (uint8_t communication_mode){

    switch (communication_mode){
        case SPI_MODE_0: // Data is sent on the rising edge
                        SPI2_CONFIG_REG1.CKE = 0x01;  
                        SPI2_CONFIG_REG1.CKP = 0x00;
                        break;
                        
        case SPI_MODE_1: 
                        SPI2_CONFIG_REG1.CKE = 0x00;  
                        SPI2_CONFIG_REG1.CKP = 0x00;      
                        break;
                        
        case SPI_MODE_2: 
                        SPI2_CONFIG_REG1.CKE = 0x01;  
                        SPI2_CONFIG_REG1.CKP = 0x01;        
                        break;
                        
        case SPI_MODE_3:
                        SPI2_CONFIG_REG1.CKE = 0x00;  
                        SPI2_CONFIG_REG1.CKP = 0x01; 
                        break;                
        }               
    }
#endif
    
#ifdef SPI3_AVAILABLE
    void SPI3_mode (uint8_t communication_mode){

    switch (communication_mode){
        case SPI_MODE_0: // Data is sent on the rising edge
                        SPI3CON1Lbits.CKE = 0x01;  
                        SPI3CON1Lbits.CKP = 0x00;
                        break;
                        
        case SPI_MODE_1: 
                        SPI3CON1Lbits.CKE = 0x00;  
                        SPI3CON1Lbits.CKP = 0x00;      
                        break;
                        
        case SPI_MODE_2: 
                        SPI3CON1Lbits.CKE = 0x01;  
                        SPI3CON1Lbits.CKP = 0x01;        
                        break;
                        
        case SPI_MODE_3:
                        SPI3CON1Lbits.CKE = 0x00;  
                        SPI3CON1Lbits.CKP = 0x01; 
                        break;                
        }               
    }
#endif    
    

//************************************************************
//               SPI1_write_byte  Implementation
//************************************************************
int8_t SPI1_write_byte (uint8_t byte_to_send){
  
    uint8_t data;    
            
    //Dummy read to clear the buffer
    data = SPI1_BUFFER;
    
    //Reset errors : Frame, Overflow, Underflow
    //Clear error history
    SPI1_FRMERR = 0x00;
    SPI1_SPIROV = 0x00;
    SPI1_SPITUR = 0x00; 	

    // Clear Interrupts flags
    SPI1_IF = 0x00;
    SPI1_TXIF = 0x00;
    SPI1_RXIF = 0x00;

    //Send the data out
    SPI1_BUFFER = byte_to_send;  

    //Wait the end of transmission
    while(SPI1_is_TX_over()); 

    //No error occurred
    return (0);                
}

#ifdef SPI2_AVAILABLE
    int8_t SPI2_write_byte (uint8_t byte_to_send){

    uint8_t data;    
            
    //Dummy read to clear the buffer
    data = SPI2_BUFFER;
    
    //Reset errors : Frame, Overflow, Underflow
    //Clear error history
    SPI2_FRMERR = 0x00;
    SPI2_SPIROV = 0x00;
    SPI2_SPITUR = 0x00; 	

    // Clear Interrupts flags
    SPI2_IF = 0x00;
    SPI2_TXIF = 0x00;
    SPI2_RXIF = 0x00; 

    //Send the data out
    SPI2_BUFFER = byte_to_send;  
    
    //Wait the end of transmission
    while(SPI2_is_TX_over()); 

    //No error occurred
    return (0);                 
    }
#endif
    
#ifdef SPI3_AVAILABLE
    int8_t SPI3_write_byte (uint8_t byte_to_send){

    uint8_t data;    
            
    //Dummy read to clear the buffer
    data = SPI3_BUFFER;
    
    //Reset errors : Frame, Overflow, Underflow
    //Clear error history
    SPI3STATLbits.FRMERR = 0x00;
    SPI3STATLbits.SPIROV = 0x00;
    SPI3STATLbits.SPITUR = 0x00; 	

    // Clear Interrupts flags
    IFS2bits.SPI2IF = 0x00;
    IFS2bits.SPI2TXIF = 0x00;
    IFS3bits.SPI2RXIF = 0x00; 

    //Send the data out
    SPI3_BUFFER = byte_to_send;  
    
    //Wait the end of transmission
    while(SPI3_is_TX_over()); 

    //No error occurred
    return (0);                 
    }
#endif    


//************************************************************
//               SPI1_read_byte  Implementation
//************************************************************
uint8_t SPI1_read_byte (void){
  
    uint8_t data;
    
    data = SPI1_BUFFER;       
      
    // Clear Interrupts flags
    SPI1_IF = 0x00;
    SPI1_TXIF = 0x00;
    SPI1_RXIF = 0x00;

    //Write 0x00 to initiate a write
    SPI1_BUFFER = 0x00;           

    //Wait end of transmission
    while(SPI1_is_TX_over());  
    
    //Return the new byte out of the 0x00 sent out
    data = SPI1_BUFFER;   
    return (data);      
}

#ifdef SPI2_AVAILABLE
    uint8_t SPI2_read_byte (void){

        uint8_t data;

        data = SPI2_BUFFER;       

        // Clear Interrupts flags
        SPI2_IF = 0x00;
        SPI2_TXIF = 0x00;
        SPI2_RXIF = 0x00;   

        //Write 0x00 to initiate a write
        SPI2_BUFFER = 0x00;           

        //Wait end of transmission
        while(SPI2_is_TX_over());  

        //Return the new byte out of the 0x00 sent out
        data = SPI2_BUFFER;   
        return (data);             
    }
#endif

#ifdef SPI3_AVAILABLE
    uint8_t SPI3_read_byte (void){

        uint8_t data;

        data = SPI3_BUFFER;       

        // Clear Interrupts flags
        IFS5bits.SPI3IF = 0x00;
        IFS5bits.SPI3TXIF = 0x00;
        IFS3bits.SPI3RXIF = 0x00;   

        //Write 0x00 to initiate a write
        SPI3_BUFFER = 0x00;           

        //Wait end of transmission
        while(SPI3_is_TX_over());  

        //Return the new byte out of the 0x00 sent out
        data = SPI3_BUFFER; 
        return (data);             
    }
#endif    
    
//************************************************************
//          SPI1_write_read_byte  Implementation
//************************************************************
uint8_t SPI1_write_read_byte (uint8_t write_byte){
  
    uint8_t data;
    
    data = SPI1_BUFFER;       
  
    //Clear interrupt flag history  
    SPI1_IF = 0x00;
    SPI1_TXIF = 0x00;
    SPI1_RXIF = 0x00;     

    //Write 0x00 to initiate a write
    SPI1_BUFFER = write_byte;           

    //Wait end of transmission
    while(SPI1_is_TX_over());  
    
    //Return the new byte out of the byte sent out
    data = SPI1_BUFFER;  
    return (data);      
}

#ifdef SPI2_AVAILABLE
    uint8_t SPI2_write_read_byte (uint8_t write_byte){

        uint8_t data;

        data = SPI2_BUFFER;       

        //Clear interrupt flag history  
        SPI2_IF = 0x00;
        SPI2_TXIF = 0x00;
        SPI2_RXIF = 0x00;   

        //Write 0x00 to initiate a write
        SPI2_BUFFER = write_byte;           

        //Wait end of transmission
        while(SPI2_is_TX_over());  

        //Return the new byte out of the byte sent out
        data = SPI2_BUFFER;
        return (data);         
    }    
#endif
    
#ifdef SPI3_AVAILABLE
    uint8_t SPI3_write_read_byte (uint8_t write_byte){

        uint8_t data;

        data = SPI3_BUFFER;       

        //Clear interrupt flag history  
        IFS5bits.SPI3IF = 0x00;
        IFS5bits.SPI3TXIF = 0x00;
        IFS3bits.SPI3RXIF = 0x00;    

        //Write 0x00 to initiate a write
        SPI3_BUFFER = write_byte;           

        //Wait end of transmission
        while(SPI3_is_TX_over());  

        //Return the new byte out of the byte sent out
        data = SPI3_BUFFER;
        return (data);         
    }    
#endif    
    