/*******************************************************************************
 
Autore : Mauro Laurenti
Versione : 1.0
 
Created on Date : 15/01/2023
Last update     : 15/01/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "module_UART.h"


//************************************************************
//         UART1_open function implementation
//************************************************************
#ifdef UART1_AVAILABLE 
void UART1_open (uint16_t baud_rate){
    
    #ifndef MCC_USED
        //IO Pin Initialization  (mux)  
        __builtin_write_OSCCONL(OSCCON & 0xbf); // unlock PPS    
            UART1_TX_LINE;
            UART1_RX_LINE;
        __builtin_write_OSCCONL(OSCCON | 0x40); // lock PPS
    #endif

    //Module Initialization
    U1STA = 0x00000000;
    U1MODE = 0x00000000;
    
    // 8 bit mode 1 stop bits no parity
    U1MODEbits.PDSEL = 0x00;
    U1MODEbits.STSEL = 0x00;
          
    //Continuous Operation in Idle Mode
    U1MODEbits.USIDL = 0x00;
    
    //Wake the Module on Start bit
    U1MODEbits.WAKE = 0x00;
    
    //TX Idle value 1
    U1STAbits.UTXINV = 0x00;    

    //Idle State 1
    UART1_RX_IDLE_STATE = 0x00;
       
    
    //Baud rate
            
    switch(baud_rate) {
        case UART_BAUDRATE_9600:
                                U1BRG = UART_BAUDRATE_9600;
                                break;
        case UART_BAUDRATE_19200:
                                U1BRG = UART_BAUDRATE_19200;
                                break;                                
        case UART_BAUDRATE_38400:
                                U1BRG = UART_BAUDRATE_38400;
                                break;
        case UART_BAUDRATE_57600:
                                U1BRG = UART_BAUDRATE_57600;
                                break;
        case UART_BAUDRATE_115200:
                                U1BRG = UART_BAUDRATE_115200;
                                break;
        case UART_BAUDRATE_250000:
                                U1BRG = UART_BAUDRATE_250000;
                                break;
        case UART_BAUDRATE_500000:
                                U1BRG = UART_BAUDRATE_500000;
                                break;
        default:
                                U1BRG = UART_BAUDRATE_9600;
    }
    
    //Turn on the UART1 Module
    U1MODEbits.UARTEN = 0x01;
    
    //Enable TX Buffer
    U1STAbits.UTXEN = 0x01;
    
    //Enable RX Buffer
    UART1_RX_EN = 0x01;
    
       
}
#endif

#ifdef UART2_AVAILABLE 

    void UART2_open (uint16_t baud_rate){
    
    #ifndef MCC_USED    
        //IO Pin Initialization  (mux)  
        __builtin_write_OSCCONL(OSCCON & 0xbf); // unlock PPS    
            UART2_TX_LINE;
            UART2_RX_LINE;
        __builtin_write_OSCCONL(OSCCON | 0x40); // lock PPS
    #endif
    
    //Module Initialization
    U2STA = 0x00000000;
    U2MODE = 0x00000000;
    
    // 8 bit mode 1 stop bits no parity
    U2MODEbits.PDSEL = 0x00;
    U2MODEbits.STSEL = 0x00;
          
    //Continuous Operation in Idle Mode
    U2MODEbits.USIDL = 0x00;
    
    //Wake the Module on Start bit
    U2MODEbits.WAKE = 0x00;
    
    //TX Idle value 1
    U2STAbits.UTXINV = 0x00;    

    //Idle State 1
    UART2_RX_IDLE_STATE = 0x00;
    
    //Baud rate
            
    switch(baud_rate) {
        case UART_BAUDRATE_9600:
                                U2BRG = UART_BAUDRATE_9600;
                                break;
        case UART_BAUDRATE_19200:
                                U2BRG = UART_BAUDRATE_19200;
                                break;                                
        case UART_BAUDRATE_38400:
                                U2BRG = UART_BAUDRATE_38400;
                                break;
        case UART_BAUDRATE_57600:
                                U2BRG = UART_BAUDRATE_57600;
                                break;
        case UART_BAUDRATE_115200:
                                U2BRG = UART_BAUDRATE_115200;
                                break;
        case UART_BAUDRATE_250000:
                                U2BRG = UART_BAUDRATE_250000;
                                break;
        case UART_BAUDRATE_500000:
                                U2BRG = UART_BAUDRATE_500000;
                                break;
        default:
                                U2BRG = UART_BAUDRATE_9600;
    }
    
    //Turn on the UART1 Module
    U2MODEbits.UARTEN = 0x01;
    
    //Enable TX Buffer
    U2STAbits.UTXEN = 0x01;
    
    //Enable RX Buffer
    UART2_RX_EN = 0x01;
    }
#endif
    
    
#ifdef UART3_AVAILABLE 

    void UART3_open (uint16_t baud_rate){
    
    #ifndef MCC_USED    
        //IO Pin Initialization  (mux)  
        __builtin_write_OSCCONL(OSCCON & 0xbf); // unlock PPS    
            UART3_TX_LINE;
            UART3_RX_LINE;
        __builtin_write_OSCCONL(OSCCON | 0x40); // lock PPS
    #endif
    
    //Module Initialization
    U3STA = 0x00000000;
    U3MODE = 0x00000000;
    
    // 8 bit mode 1 stop bits no parity
    U3MODEbits.PDSEL = 0x00;
    U3MODEbits.STSEL = 0x00;
          
    //Continuous Operation in Idle Mode
    U3MODEbits.USIDL = 0x00;
    
    //Wake the Module on Start bit
    U3MODEbits.WAKE = 0x00;
    
    //TX Idle value 1
    U3STAbits.UTXINV = 0x00;    

    //Idle State 1
    UART3_RX_IDLE_STATE = 0x00;
    
    
    //Baud rate
            
    switch(baud_rate) {
        case UART_BAUDRATE_9600:
                                U3BRG = UART_BAUDRATE_9600;
                                break;
        case UART_BAUDRATE_19200:
                                U3BRG = UART_BAUDRATE_19200;
                                break;                                
        case UART_BAUDRATE_38400:
                                U3BRG = UART_BAUDRATE_38400;
                                break;
        case UART_BAUDRATE_57600:
                                U3BRG = UART_BAUDRATE_57600;
                                break;
        case UART_BAUDRATE_115200:
                                U3BRG = UART_BAUDRATE_115200;
                                break;
        case UART_BAUDRATE_250000:
                                U3BRG = UART_BAUDRATE_250000;
                                break;
        case UART_BAUDRATE_500000:
                                U3BRG = UART_BAUDRATE_500000;
                                break;
        default:
                                U3BRG = UART_BAUDRATE_9600;
    }
    
    //Turn on the UART1 Module
    U3MODEbits.UARTEN = 0x01;
    
    //Enable TX Buffer
    U3STAbits.UTXEN = 0x01;
    
    //Enable RX Buffer
    UART3_RX_EN = 0x01;
    }
#endif    


#ifdef UART4_AVAILABLE 

    void UART4_open (uint16_t baud_rate){
    
    #ifndef MCC_USED    
        //IO Pin Initialization  (mux)  
        __builtin_write_OSCCONL(OSCCON & 0xbf); // unlock PPS    
            UART4_TX_LINE;
            UART4_RX_LINE;
        __builtin_write_OSCCONL(OSCCON | 0x40); // lock PPS
    #endif
    
    //Module Initialization
    U4STA = 0x00000000;
    U4MODE = 0x00000000;
    
    // 8 bit mode 1 stop bits no parity
    U4MODEbits.PDSEL = 0x00;
    U4MODEbits.STSEL = 0x00;
          
    //Continuous Operation in Idle Mode
    U4MODEbits.USIDL = 0x00;
    
    //Wake the Module on Start bit
    U4MODEbits.WAKE = 0x00;
    
    //TX Idle value 1
    U4STAbits.UTXINV = 0x00;    

    //Idle State 1
    UART4_RX_IDLE_STATE = 0x00;
    
    
    //Baud rate
            
    switch(baud_rate) {
        case UART_BAUDRATE_9600:
                                U4BRG = UART_BAUDRATE_9600;
                                break;
        case UART_BAUDRATE_19200:
                                U4BRG = UART_BAUDRATE_19200;
                                break;                                
        case UART_BAUDRATE_38400:
                                U4BRG = UART_BAUDRATE_38400;
                                break;
        case UART_BAUDRATE_57600:
                                U4BRG = UART_BAUDRATE_57600;
                                break;
        case UART_BAUDRATE_115200:
                                U4BRG = UART_BAUDRATE_115200;
                                break;
        case UART_BAUDRATE_250000:
                                U4BRG = UART_BAUDRATE_250000;
                                break;
        case UART_BAUDRATE_500000:
                                U4BRG = UART_BAUDRATE_500000;
                                break;
        default:
                                U4BRG = UART_BAUDRATE_9600;
    }
    
    //Turn on the UART1 Module
    U4MODEbits.UARTEN = 0x01;
    
    //Enable TX Buffer
    U4STAbits.UTXEN = 0x01;
    
    //Enable RX Buffer
    UART4_RX_EN = 0x01;
    }
#endif    
    
    
//************************************************************
//         UART1_write_byte function implementation
//************************************************************
#ifdef UART1_AVAILABLE 
    void UART1_write_byte (uint8_t data) { 
    
    //wait until there is something free in the buffer
    while(U1STAbits.UTXBF == 1);

    // Write the data into the TX buffer.
    U1TXREG = data;    
      
    while (UART1_TX_busy( ));
    }
#endif

#ifdef UART2_AVAILABLE 
    void UART2_write_byte (uint8_t data) { 
    
        //wait until there is something free in the buffer
        while(U2STAbits.UTXBF == 1);

        // Write the data into the TX buffer.
        U2TXREG = data;    

        while (UART2_TX_busy( ));
    }
#endif
    
#ifdef UART3_AVAILABLE 
    void UART3_write_byte (uint8_t data) { 
    
        //wait until there is something free in the buffer
        while(U3STAbits.UTXBF == 1);

        // Write the data into the TX buffer.
        U3TXREG = data;    

        while (UART3_TX_busy( ));
    }
#endif    
    
#ifdef UART4_AVAILABLE 
    void UART4_write_byte (uint8_t data) { 
    
        //wait until there is something free in the buffer
        while(U4STAbits.UTXBF == 1);

        // Write the data into the TX buffer.
        U4TXREG = data;    

        while (UART4_TX_busy( ));
    }
#endif    
    
//************************************************************
//         UART1_write_message function implementation
//************************************************************
#ifdef UART1_AVAILABLE 
    void UART1_write_message(const char *data){

        while(*data) {	    
            UART1_write_byte(*data);
            while(UART1_TX_busy());
            data++;              
        }  
    }
#endif

#ifdef UART2_AVAILABLE 
    void UART2_write_message(const char *data){

        while(*data) {	    
            UART2_write_byte(*data);
            while(UART2_TX_busy());
            data++;              
        }  
    }
#endif
    
#ifdef UART3_AVAILABLE 
    void UART3_write_message(const char *data){

        while(*data) {	    
            UART3_write_byte(*data);
            while(UART3_TX_busy());
            data++;              
        }  
    }
#endif

#ifdef UART4_AVAILABLE 
    void UART4_write_message(const char *data){

        while(*data) {	    
            UART4_write_byte(*data);
            while(UART4_TX_busy());
            data++;              
        }  
    }
#endif     

//************************************************************
//         UART1_write_string function implementation
//************************************************************
#ifdef UART1_AVAILABLE 
    void UART1_write_string(uint8_t *data){

        while(*data) {	    
            UART1_write_byte(*data);
            while(UART1_TX_busy());
            data++;              
        }  
    }
#endif

#ifdef UART2_AVAILABLE 
    void UART2_write_string(uint8_t *data){

        while(*data) {	    
            UART2_write_byte(*data);
            while(UART2_TX_busy());
            data++;              
        }  
    }
#endif
    
#ifdef UART3_AVAILABLE 
    void UART3_write_string(uint8_t *data){

        while(*data) {	    
            UART3_write_byte(*data);
            while(UART3_TX_busy());
            data++;              
        }  
    }
#endif 
    
#ifdef UART4_AVAILABLE 
    void UART4_write_string(uint8_t *data){

        while(*data) {	    
            UART4_write_byte(*data);
            while(UART4_TX_busy());
            data++;              
        }  
    }
#endif       
    
//************************************************************
//         UART1_read_byte function implementation
//************************************************************
#ifdef UART1_AVAILABLE 
    uint8_t UART1_read_byte(void){

    uint8_t data;  
      
    if ((U1STAbits.OERR == 1)){
        U1STAbits.OERR = 0;
    }

    data = U1RXREG;                      

    return (data);   
      
    }
#endif

#ifdef UART2_AVAILABLE 
    uint8_t UART2_read_byte(void){

    uint8_t data;  
      
    if ((U2STAbits.OERR == 1)){
        U2STAbits.OERR = 0;
    }

    data = U2RXREG;                      

    return (data);                       
    }
#endif
    
#ifdef UART3_AVAILABLE 
    uint8_t UART3_read_byte(void){

    uint8_t data;  
      
    if ((U3STAbits.OERR == 1)){
        U3STAbits.OERR = 0;
    }

    data = U3RXREG;                      

    return (data);                       
    }
#endif    
    
#ifdef UART4_AVAILABLE 
    uint8_t UART4_read_byte(void){

    uint8_t data;  
      
    if ((U4STAbits.OERR == 1)){
        U4STAbits.OERR = 0;
    }

    data = U4RXREG;                      

    return (data);                       
    }
#endif        
    