/*******************************************************************************
 
Autore : Mauro Laurenti
Versione : 1.0
 
Created on Date : 18/09/2019
Last update     : 18/09/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
*******************************************************************************/

#ifdef __XC8
    #include <xc.h>
#endif

#include "module_ADC.h"

static uint16_t acquisition_time_us = 100;

//************************************************************
//         ADC_open function implementation
//************************************************************
void ADC_open (uint32_t ADC_enable_channel){
       
    //Enable the Port Pin for the ADC Channel
    #if (ADC_NUMBER_OF_CHANNELS == 10)
        if (ADC_enable_channel & ADC_CH0) ADC_PORT_CONFIGURATION_1 |= 0x0001; 
        if (ADC_enable_channel & ADC_CH1) ADC_PORT_CONFIGURATION_1 |= 0x0002; 
        if (ADC_enable_channel & ADC_CH2) ADC_PORT_CONFIGURATION_2 |= 0x0001;  
        if (ADC_enable_channel & ADC_CH3) ADC_PORT_CONFIGURATION_2 |= 0x0002;   
        if (ADC_enable_channel & ADC_CH4) ADC_PORT_CONFIGURATION_2 |= 0x0004;  
        if (ADC_enable_channel & ADC_CH5) ADC_PORT_CONFIGURATION_2 |= 0x0008;  
        if (ADC_enable_channel & ADC_CH9) ADC_PORT_CONFIGURATION_2 |= 0x8000; 
        if (ADC_enable_channel & ADC_CH10) ADC_PORT_CONFIGURATION_2|= 0x4000;
        if (ADC_enable_channel & ADC_CH11) ADC_PORT_CONFIGURATION_2 |= 0x2000; 
        if (ADC_enable_channel & ADC_CH12) ADC_PORT_CONFIGURATION_2 |= 0x1000; 
    #endif

    #if (ADC_NUMBER_OF_CHANNELS == 12 || ADC_NUMBER_OF_CHANNELS == 13)
        if (ADC_enable_channel & ADC_CH0) ADC_PORT_CONFIGURATION_1 |= 0x0001; 
        if (ADC_enable_channel & ADC_CH1) ADC_PORT_CONFIGURATION_1 |= 0x0002; 
        if (ADC_enable_channel & ADC_CH2) ADC_PORT_CONFIGURATION_2 |= 0x0001;  
        if (ADC_enable_channel & ADC_CH3) ADC_PORT_CONFIGURATION_2 |= 0x0002;   
        if (ADC_enable_channel & ADC_CH4) ADC_PORT_CONFIGURATION_2 |= 0x0004;  
        if (ADC_enable_channel & ADC_CH5) ADC_PORT_CONFIGURATION_2 |= 0x0008;  
        if (ADC_enable_channel & ADC_CH6) ADC_PORT_CONFIGURATION_3 |= 0x0001;
        if (ADC_enable_channel & ADC_CH7) ADC_PORT_CONFIGURATION_3 |= 0x0002;
        if (ADC_enable_channel & ADC_CH8) ADC_PORT_CONFIGURATION_3 |= 0x0008;
        if (ADC_enable_channel & ADC_CH9) ADC_PORT_CONFIGURATION_2 |= 0x8000; 
        if (ADC_enable_channel & ADC_CH10) ADC_PORT_CONFIGURATION_2 |= 0x4000;
        if (ADC_enable_channel & ADC_CH11) ADC_PORT_CONFIGURATION_2 |= 0x2000; 
        if (ADC_enable_channel & ADC_CH12) ADC_PORT_CONFIGURATION_2 |= 0x1000; 
    #endif

    ADC_set_reference (ADC_VREF_P_AVDD_N_AVSS);
    ADC_set_acquisition_time(1000);

    ADC_set_clock(ADC_CLOCK_PBCLK);
	
	ADC_CONVERSION_DONE = 0x00;
    
    ADC_ENABLE_BIT |= 0x01;
}


//************************************************************
//         ADC_close function implementation
//************************************************************
void ADC_close (void){
    ADC_ENABLE_BIT = 0x00;  

    //Disable all the ADC channels to reduce the leackage  
    #if (ADC_NUMBER_OF_CHANNELS == 10)
        ADC_PORT_CONFIGURATION_1 = 0x00000000;
        ADC_PORT_CONFIGURATION_2 = 0x00000000;
    #endif

    #if (ADC_NUMBER_OF_CHANNELS == 12 || ADC_NUMBER_OF_CHANNELS == 13)
        ADC_PORT_CONFIGURATION_1 = 0x00000000;
        ADC_PORT_CONFIGURATION_2 = 0x00000000;
        ADC_PORT_CONFIGURATION_3 = 0x00000000;
    #endif    
}


//************************************************************
//         ADC_select_channel function implementation
//************************************************************
void ADC_select_channel (uint32_t ADC_channel){
    
    ADC_CHANNEL_SELECTION_REGISTER = ADC_channel_converter (ADC_channel)<< 16;
}


//************************************************************
//         ADC_start_conversion function implementation
//************************************************************
void ADC_start_conversion (void){
    ADC_START_CONVERSION_BIT = 0x01;
    delay_us(acquisition_time_us);
    ADC_START_CONVERSION_BIT = 0x00;
}


//************************************************************
//         ADC_abort_conversion function implementation
//************************************************************
void ADC_abort_conversion (void){
    ADC_START_CONVERSION_BIT = 0x00;
    ADC_STOP_CONVERSION_BIT = 0x00;
}


//************************************************************
//         ADC_abort_conversion function implementation
//************************************************************
uint16_t ADC_read_value (void){
    uint16_t ADC_value = 0;
    
    ADC_value = (uint16_t) ADC_BUFFER;
 
    return (ADC_value);
}


//************************************************************
//         ADC_is_converting function implementation
//************************************************************
uint8_t ADC_is_converting (){
    if (ADC_CONVERSION_DONE == 0){
        return (1);
    } else {
        //Reset the interrupt flag bit for a new conversion
        ADC_CONVERSION_DONE = 0x00;
        return (0);
    }
}


//************************************************************
//         ADC_set_acquisition_time function implementation
//************************************************************
void ADC_set_acquisition_time (register_size acq_time){
    
    acquisition_time_us = acq_time;
}


//************************************************************
//         ADC_set_clock function implementation
//************************************************************
void ADC_set_clock (register_size ADC_clock){
   
   ADC_SETTINGS_3 &= ~ADC_CLOCK_RESET_MASK;
   ADC_SETTINGS_3 |= ADC_clock;
}



//************************************************************
//         ADC_set_reference function implementation
//************************************************************
void ADC_set_reference (register_size ADC_reference){
    
    ADC_SETTINGS_2 |= ADC_reference; 
}


//************************************************************
//         ADC_read_channel function implementation
//************************************************************
uint16_t ADC_read_channel (uint32_t ADC_channel){
    
    uint16_t ADC_value = 0;
    
    ADC_open(ADC_channel);
    ADC_select_channel (ADC_channel);

    ADC_start_conversion();

    while(ADC_is_converting());

    ADC_value = ADC_read_value();
    
    ADC_close();
    
    return(ADC_value);    
}


//************************************************************
//         ADC_channel_converter function implementation
//************************************************************
uint8_t ADC_channel_converter (uint32_t channel_mask){

    uint32_t filter = 0x00000001;
    uint8_t counter = 0;
    uint8_t adc_channel = 0;

    for (counter=0; counter < ADC_NUMBER_OF_CHANNELS; counter++){
        if(channel_mask & filter){
            adc_channel = counter;
        }
        filter = filter << 1;
    }
    return (adc_channel);
}
