/*******************************************************************************
 
Author : Mauro Laurenti
Version : 1.1
 
Created on Date : 06/11/2018
Last update     : 31/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "module_FLASH.h"


//************************************************************
//
//************************************************************
void FLASH_read_memory (uint32_t memory_address, uint32_t block_size, uint32_t *buffer){
    
    uint32_t memory_offset = 0;
    uint32_t data_readback = 0;
    uint8_t word_width = 4;
    uint32_t counter = 0;
    
    for (counter=0; counter < block_size; counter++){
        
        //For the first cycle memory offset it not added.
        if (counter > 0){
            memory_offset = (memory_offset * counter) + word_width;
        } 
        
        data_readback = *(uint32_t*)(memory_address + memory_offset);

        buffer[counter] = data_readback;        
    }
}

//************************************************************
//
//************************************************************
uint32_t FLASH_read_device_ID (void){
    uint32_t device_info;
    uint32_t device_ID = 0;
    
    FLASH_read_memory (DEVICE_ID_ADDRESS, DEVICE_ID_WORDS, &device_info);
    
    device_ID |= device_info & 0x0FFFFFFF;
    
    return (device_ID);
}


//************************************************************
//
//************************************************************
uint32_t FLASH_read_device_revision (void){

    uint32_t device_info;
    uint32_t device_revision = 0;
    
    FLASH_read_memory (DEVICE_REVISION_ADDRESS, DEVICE_REVISION_WORDS, &device_info);
    
    device_revision |= (device_info & 0xF0000000) >> 28;
    
    return (device_revision);
}


//************************************************************
//
//************************************************************
int8_t FLASH_erase_memory (uint32_t memory_address){
    
    //Memory page setting
    NVMADDR = memory_address;
    
    //Configure the memory controller for the page erase and enable write operations
    NVMCON = 0x00004004;
    
    // Write Keys to unlock the controller
    NVMKEY = FLASH_KEY_1;
    NVMKEY = FLASH_KEY_2;
    
    // Start the erase operation
    NVMCONSET = 0x8000;
    
    // Wait for the operation to complete
    while (NVMCON & 0x8000);
    
    // Disable write operations
    NVMCONCLR = 0x0004000;
    
    //Check for any errors    
    if (NVMCON & 0x3000 != 0){
        return (-1);
    } else {
         return (0);
    }    
}


//************************************************************
//
//************************************************************
int8_t FLASH_write_word (uint32_t memory_address, uint32_t data){
        
    //Word to write
    NVMDATA = data;
    
    //Memory address setting
    NVMADDR = memory_address;
        
    //Configure the memory controller for the word write and enable write operations
    NVMCON = 0x00004001;
    
    // Write Keys to unlock the controller
    NVMKEY = FLASH_KEY_1;
    NVMKEY = FLASH_KEY_2;
    
    // Start the write operation
    NVMCONSET = 0x8000;
    
    // Wait for the operation to complete
    while (NVMCON & 0x8000);
    
    // Disable write operations
    NVMCONCLR = 0x0004000;
    
    //Check for any errors    
    if (NVMCON & 0x3000 != 0){
        return (-1);
    } else {
         return (0);
    }  
}

//************************************************************
//
//************************************************************
int8_t FLASH_write_memory (uint32_t memory_address, uint16_t block_size, uint32_t *buffer){
    
    //Check memory size
    if (block_size > FLASH_WRITE_BLOCK){
        return (-2);
    }
            
    //Memory start address (FLASH)
    NVMADDR = memory_address;
        
    //Set the data memory block start address (RAM))
    NVMSRCADDR = (uint32_t) buffer;
    
    //Configure the memory controller for the row write and enable write operations
    NVMCON = 0x4003;
    
    // Write Keys to unlock the controller
    NVMKEY = FLASH_KEY_1;
    NVMKEY = FLASH_KEY_2;
    
    // Start the write operation
    NVMCONSET = 0x8000;
    
    // Wait for the operation to complete
    while (NVMCON & 0x8000);
    
    // Disable write operations
    NVMCONCLR = 0x0004000;
    
    //Check for any errors    
    if (NVMCON & 0x3000 != 0){
        return (-1);
    } else {
         return (0);
    }  
}
 