/*******************************************************************************
 
Autore : Mauro Laurenti
Versione : 1.4
 
Created on Date : 02/10/2010
Last update     : 09/02/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "module_I2C.h"

//************************************************************
//         I2C1_open function implementation
//************************************************************

void I2C1_open (uint8_t device_type){
  
    uint8_t dummy_read;
    
    //IO pin initialization
    //It is not required since the module will overwrite the TRIS settings.
        
    
    //Register initialization (Starting known value)
    //Reset error
    //Bus Collision
    I2C1STATbits.BCL = 0x00; 
    //Write Collision 
    I2C1STATbits.IWCOL = 0x00;
    //Receive Overflow
    I2C1STATbits.I2COV = 0x00;
    
    //Works in Idle mode
    I2C1CONbits.SIDL = 0x00;
    
    //Device 7 10 bits configuration 0 = 7bits  and 1 = 10bits
    I2C1CONbits.A10M = 0x00;
    
    //Device definition Master or Slave   
    //PIC32 can work as master and slave simultaneously
    
    
    //Slew Rate Control
    if(device_type & I2C_SLEW_OFF){
        I2C1CONbits.DISSLW = 0x01;
    } else {
        I2C1CONbits.DISSLW = 0x00;
    }
      
    //Enable the I2C Module
    I2C1CONbits.ON = 0x01;    
    
    //clear input buffer
    dummy_read = I2C1_read_byte ();
    
}

#ifdef I2C2_AVAILABLE 

void I2C2_open (uint8_t device_type){
    
    uint8_t dummy_read;
    
    //IO pin initialization
    //It is not required since the module will overwrite the TRIS settings.
        
    
    //Register initialization (Starting known value)
    //Reset error
    //Bus Collision
    I2C2STATbits.BCL = 0x00; 
    //Write Collision 
    I2C2STATbits.IWCOL = 0x00;
    //Receive Overflow
    I2C2STATbits.I2COV = 0x00;
    
    //Works in Idle mode
    I2C2CONbits.SIDL = 0x00;
    
    //Device 7 10 bits configuration 0 = 7bits  and 1 = 10bits
    I2C2CONbits.A10M = 0x00;
    
    //Device definition Master or Slave   
    //PIC32 can work as master ans slave simultaneously
    
    
    //Slew Rate Control
    if(device_type & I2C_SLEW_OFF){
        I2C2CONbits.DISSLW = 0x01;
    } else {
        I2C2CONbits.DISSLW = 0x00;
    }
    
    I2C2CONbits.DISSLW = 0x00;
    
    //Enable the I2C Module
    I2C2CONbits.ON = 0x01;    
    
    //clear input buffer
    dummy_read = I2C2_read_byte ();
    
}
#endif

//************************************************************
//         I2C1_close function implementation
//************************************************************
void I2C1_close (void){
    I2C1CONbits.ON = 0x00; 
}

#ifdef I2C2_AVAILABLE 
    void I2C2_close (void){
        I2C2CONbits.ON = 0x00; 
    }
#endif

//************************************************************
//         I2C1_baud_rate function implementation
//************************************************************
void I2C1_baud_rate (uint16_t bus_baud_rate_KHZ){
    
    //The formula is simplified without the Tpgd correction
    uint16_t div_a = (uint16_t) (I2C_CLOCK /1000);
    uint16_t div_b = (2 *bus_baud_rate_KHZ);    
    I2C1BRG = (uint16_t)(div_a/div_b)-2;
}

#ifdef I2C2_AVAILABLE 
void I2C2_baud_rate (uint16_t bus_baud_rate_KHZ){
    //The formula is simplified without the Tpgd correction
    uint16_t div_a = (uint16_t) (I2C_CLOCK /1000);
    uint16_t div_b = (2 *bus_baud_rate_KHZ);    
    I2C2BRG = (uint16_t)(div_a/div_b)-2;
}
#endif

//************************************************************
//      I2C1_set_slave_address function implementation
//************************************************************
void I2C1_set_slave_address (uint16_t device_address){

    I2C1ADD = device_address;
}

#ifdef I2C2_AVAILABLE 
void I2C2_set_slave_address (uint16_t device_address){
    I2C2ADD = device_address;
}
#endif

//************************************************************
//         I2C1_wait_bus_IDLE function implementation
//************************************************************
void I2C1_wait_bus_IDLE(void){
    uint8_t idle_flag =0x01;
    
    //If any of the flag below is 1, some activieties are still ongoing
    //If at the end of the tests idle_flag is still 0, the bus is Idle
    while (idle_flag){
        idle_flag = 0;
        
        idle_flag |= I2C1STATbits.TBF;
        idle_flag |= I2C1STATbits.ACKSTAT;
        idle_flag |= I2C1STATbits.TRSTAT;
        idle_flag |= I2C1CONbits.RCEN;
        idle_flag |= I2C1CONbits.PEN;
        idle_flag |= I2C1CONbits.RSEN;
        idle_flag |= I2C1CONbits.SEN;
    }
}

#ifdef I2C2_AVAILABLE 
void I2C2_wait_bus_IDLE(void){
    uint8_t idle_flag =0x01;
    
    //If any of the flag below is 1, some activities are still ongoing
    //If at the end of the tests idle_flag is still 0, the bus is Idle
    while (idle_flag){
        idle_flag = 0;
        
        idle_flag |= I2C2STATbits.TBF;
        idle_flag |= I2C2STATbits.ACKSTAT;
        idle_flag |= I2C2STATbits.TRSTAT;
        idle_flag |= I2C2CONbits.RCEN;
        idle_flag |= I2C2CONbits.PEN;
        idle_flag |= I2C2CONbits.RSEN;
        idle_flag |= I2C2CONbits.SEN;
    }
}
#endif   


//************************************************************
//         I2C1_start_bit function implementation
//************************************************************
void I2C1_start_bit (void){
    
    I2C1CONbits.SEN=1;
    //Waits until start bit is in idle
    while(I2C1CONbits.SEN);
}

 
#ifdef I2C2_AVAILABLE 
void I2C2_start_bit (void){
    I2C2CONbits.SEN=1;
    //Waits until start bit is in idle
    while(I2C2CONbits.SEN);
}
#endif   
    

//************************************************************
//         I2C1_stop_bit function implementation
//************************************************************  
    
void I2C1_stop_bit (void){
    I2C1CONbits.PEN=1;
    while(I2C1CONbits.PEN);
}

 
#ifdef I2C2_AVAILABLE 
void I2C2_stop_bit (void){

    I2C2CONbits.PEN=1;
    while(I2C2CONbits.PEN);
}
#endif 

//************************************************************
//         I2C1_negative_ACK function implementation
//************************************************************  
void I2C1_negative_ACK(void){
    I2C1CONbits.ACKDT=1; 
    I2C1CONbits.ACKEN=1;
    while(I2C1CONbits.ACKEN);
}

#ifdef I2C2_AVAILABLE 
    void I2C2_negative_ACK(void){
        
        I2C2CONbits.ACKDT=1; 
        I2C2CONbits.ACKEN=1;
        while(I2C2CONbits.ACKEN);    
    }
    
#endif   
    
    
//************************************************************
//         I2C1_negative_ACK function implementation
//************************************************************  
void I2C1_ACK(void){
    I2C1CONbits.ACKDT=0; 
    I2C1CONbits.ACKEN=1;
    while(I2C1CONbits.ACKEN);
}

#ifdef I2C2_AVAILABLE 
    void I2C2_ACK(void){
        
        I2C2CONbits.ACKDT=0; 
        I2C2CONbits.ACKEN=1;
        while(I2C2CONbits.ACKEN);    
    }
    
#endif       

   
//************************************************************
//         I2C1_write_byte function implementation
//************************************************************      
int8_t I2C1_write_byte (uint8_t byte_to_send){
 
    //Reset error
    //Bus Collision
    I2C1STATbits.BCL = 0x00; 
    //Write Collision 
    I2C1STATbits.IWCOL = 0x00;
    //Receive Overflow
    I2C1STATbits.I2COV = 0x00;
    
    
    //Writes the data to the output buffer
    I2C1TRN = byte_to_send;           
    
    //Check for collision errors
    if (I2C1STATbits.IWCOL)      
      return ( -1 );              
    else {
    
    //slave mode only 
//    if( ((SSPCON1&0x0F)!=0x08) && ((SSPCON1&0x0F)!=0x0B) ) {
//
//      // release clock line   
//      SSPCON1bits.CKP = 1;       
//      
//      // wait until the nineth clock pulse is received
//      while ( !PIR1bits.SSPIF );  
//      
//      // if R/W=0 and BF=0, NOT ACK was received
//      if ( ( !SSPSTATbits.R_W ) && ( !SSPSTATbits.BF )){
//        return ( -2 );             
//      }	
//	  else 
//          return(0);		
//	
//    }
   
        //master mode only	
        // wait until write cycle is complete    
        while(I2C1STATbits.TBF);   

        I2C1_wait_bus_IDLE();                  

        // test for ACK condition received
        if (I2C1STATbits.ACKSTAT ) 
             return ( -2 );				
        else 
            return ( 0 );             
    }
  
  //Unknown Error  
  return ( -3 );  
}

#ifdef I2C2_AVAILABLE 
int8_t I2C2_write_byte (uint8_t byte_to_send){

    //Reset error
    //Bus Collision
    I2C2STATbits.BCL = 0x00; 
    //Write Collision 
    I2C2STATbits.IWCOL = 0x00;
    //Receive Overflow
    I2C2STATbits.I2COV = 0x00;
    
    //Writes the data to the output buffer
    I2C2TRN = byte_to_send;           
    
    //Check for collision errors
    if (I2C2STATbits.IWCOL)      
      return ( -1 );              
    else {
    
    //slave mode only 
//    if( ((SSPCON1&0x0F)!=0x08) && ((SSPCON1&0x0F)!=0x0B) ) {
//
//      // release clock line   
//      SSPCON1bits.CKP = 1;       
//      
//      // wait until the nineth clock pulse is received
//      while ( !PIR1bits.SSPIF );  
//      
//      // if R/W=0 and BF=0, NOT ACK was received
//      if ( ( !SSPSTATbits.R_W ) && ( !SSPSTATbits.BF )){
//        return ( -2 );             
//      }	
//	  else 
//          return(0);		
//	
//    }

        
    //master mode only	    
      
    // wait until write cycle is complete    
    while(I2C2STATbits.TBF);   

    I2C2_wait_bus_IDLE();                  

    // test for ACK condition received
    if (I2C2STATbits.ACKSTAT ) 
         return ( -2 );				
    else 
        return ( 0 );             

  }
  
  //Unknown Error  
  return ( -3 );  
}
#endif        
    

//************************************************************
//     I2C1_reset_write_collision_flag function implementation
//************************************************************     
void I2C1_reset_write_collision_flag (void){
    I2C1STATbits.IWCOL=0;
}
 
#ifdef I2C2_AVAILABLE 
void I2C2_reset_write_collision_flag (void){
    I2C2STATbits.IWCOL=0;      
}
#endif   


//************************************************************
//   I2C1_write_byte_to_external_device function implementation
//************************************************************  
int8_t I2C1_write_byte_to_external_device (uint8_t control_byte, uint8_t register_address, uint8_t data){
  
    I2C1_wait_bus_IDLE();                     
    I2C1_start_bit();                     
  
    // test for bus collision
    if (I2C1STATbits.BCL){
        return ( -1 );                
    }
    else {
        // write byte - R/W bit should be 0    
        if ( I2C1_write_byte(control_byte)){
          I2C1_stop_bit(); 
          return (-3);              
        }

        // test for ACK condition received
        if ( !I2C1STATbits.ACKSTAT ){
            
          if (I2C1_write_byte(register_address)) {
            I2C1_stop_bit(); 
            return ( -3 );            
          }
        // test for ACK condition received
        if (!I2C1STATbits.ACKSTAT){ 
            if (I2C1_write_byte(data)) {
              I2C1_stop_bit(); 
              return ( -3 );          
            }
        }else {
            I2C1_stop_bit(); 
            return ( -2 );            
          }
        } else {
          I2C1_stop_bit(); 
          return ( -2 );              
        }
    }

    I2C1_stop_bit();                      

    // test for bus collision
    if (I2C1STATbits.BCL){
        return ( -1 );                
    }
    
    return ( 0 );                   
}
 
#ifdef I2C2_AVAILABLE 
int8_t  I2C2_write_byte_to_external_device (uint8_t control_byte, uint8_t register_address, uint8_t data){
    
    I2C2_wait_bus_IDLE();                     
    I2C2_start_bit();                     
  
    // test for bus collision
    if (I2C2STATbits.BCL){
        return ( -1 );                
    }
    else {
        // write byte - R/W bit should be 0    
        if ( I2C2_write_byte(control_byte)){
          I2C2_stop_bit(); 
          return (-3);              
        }

        // test for ACK condition received
        if ( !I2C2STATbits.ACKSTAT ){
            
          if (I2C2_write_byte(register_address)) {
            I2C2_stop_bit(); 
            return ( -3 );            
          }
        // test for ACK condition received
        if (!I2C2STATbits.ACKSTAT){ 
            if (I2C2_write_byte(data)) {
              I2C2_stop_bit(); 
              return ( -3 );          
            }
        }else {
            I2C2_stop_bit(); 
            return ( -2 );            
          }
        } else {
          I2C2_stop_bit(); 
          return ( -2 );              
        }
    }

    I2C2_stop_bit();                      

    // test for bus collision
    if (I2C2STATbits.BCL){
        return ( -1 );                
    }
    
    return ( 0 );         
}
#endif  

//************************************************************
//   I2C1_read_byte_from_external_device function implementation
//************************************************************  

int8_t I2C1_read_byte_from_external_device (uint8_t control_byte, uint8_t register_address, uint8_t *data){
   
    I2C1_wait_bus_IDLE();                     
    I2C1_start_bit();                     

    // test for bus collision
    if (I2C1STATbits.BCL){
        return (-1);                
    } else {
        if (I2C1_write_byte(control_byte)) {
          I2C1_stop_bit(); 
          return (-3);             
        }
        
        // Check for ACK condition
        if (!I2C1STATbits.ACKSTAT){
           // WRITE word address for EEPROM 
          if (I2C1_write_byte(register_address)){
            I2C1_stop_bit(); 
            return (-3);            
          }
        
          // Check for ACK condition
          if (!I2C1STATbits.ACKSTAT) {
              
            I2C1_restart_communication();            
            
            // test for bus collision
            if (I2C1STATbits.BCL){
              return (-1);         
            }

            // write 1 byte - R/W bit should be 1
            if (I2C1_write_byte(control_byte+1)){
              I2C1_stop_bit(); 
              return (-3);         
            }

            // Check for ACK condition
            if (!I2C1STATbits.ACKSTAT){
              // enable master for 1 byte reception  
              I2C1CONbits.RCEN = 1;   
              // check that receive sequence is over
              while (I2C1CONbits.RCEN); 

              // send ACK condition
              I2C1_negative_ACK();    
              
              // send STOP condition
              I2C1_stop_bit();             

              // test for bus collision
              if (I2C1STATbits.BCL) {
               return (-1);         
              }
              
            } else {
              I2C1_stop_bit(); 
              return (-2);          
            }
          } else {
            I2C1_stop_bit(); 
            return (-2);           
          }
        }
        else
        {
          I2C1_stop_bit(); 
          return (-2);              
        }
    }
    
    // if no error occured the data is valid!
    *data = I2C1RCV;
    
    return (0);     
}
 
#ifdef I2C2_AVAILABLE 
int8_t  I2C2_read_byte_from_external_device (uint8_t control_byte, uint8_t register_address, uint8_t *data){
    I2C2_wait_bus_IDLE();                     
    I2C2_start_bit();                     

    // test for bus collision
    if (I2C2STATbits.BCL){
        return (-1);                
    } else {
        if (I2C2_write_byte(control_byte)) {
          I2C2_stop_bit(); 
          return (-3);             
        }
        
        // Check for ACK condition
        if (!I2C2STATbits.ACKSTAT){
           // WRITE word address for EEPROM 
          if (I2C2_write_byte(register_address)){
            I2C2_stop_bit(); 
            return (-3);            
          }
        
          // Check for ACK condition
          if (!I2C2STATbits.ACKSTAT) {
              
            I2C2_restart_communication();            
            
            // test for bus collision
            if (I2C2STATbits.BCL){
              return (-1);         
            }

            // write 1 byte - R/W bit should be 1
            if (I2C2_write_byte(control_byte+1)){
              I2C2_stop_bit(); 
              return (-3);         
            }

            // Check for ACK condition
            if (!I2C2STATbits.ACKSTAT){
              // enable master for 1 byte reception  
              I2C2CONbits.RCEN = 1;   
              // check that receive sequence is over
              while (I2C2CONbits.RCEN); 

              // send ACK condition
              I2C2_negative_ACK();    
              
              // send STOP condition
              I2C2_stop_bit();             

              // test for bus collision
              if (I2C2STATbits.BCL) {
               return (-1);         
              }
            } else {
              I2C2_stop_bit(); 
              return (-2);          
            }
          } else {
            I2C2_stop_bit(); 
            return (-2);           
          }
        }
        else
        {
          I2C2_stop_bit(); 
          return (-2);              
        }
    }
    
    // if no error occured the data is valid!
    *data = I2C2RCV;
    
    return (0);  
}

#endif  


//************************************************************
//   I2C1_write_word_to_external_device function implementation
//************************************************************  
int8_t I2C1_write_word_to_external_device (uint8_t control_byte, uint8_t register_address, uint16_t data){
  
    I2C1_wait_bus_IDLE();                     
    I2C1_start_bit();                     
  
    // test for bus collision
    if (I2C1STATbits.BCL){
        return ( -1 );                
    }
    else {
        // write byte - R/W bit should be 0    
        if ( I2C1_write_byte(control_byte)){
          I2C1_stop_bit(); 
          return (-3);              
        }

        // test for ACK condition received
        if ( !I2C1STATbits.ACKSTAT ){
            
          if (I2C1_write_byte(register_address)) {
            I2C1_stop_bit(); 
            return ( -3 );            
          }
          
        // test for ACK condition received
        if (!I2C1STATbits.ACKSTAT){ 
            if (I2C1_write_byte((uint8_t) (data >> 8))) {
              I2C1_stop_bit(); 
              return ( -3 );          
            }
        }else {
            I2C1_stop_bit(); 
            return ( -2 );            
        }
        
        // test for ACK condition received
        if (!I2C1STATbits.ACKSTAT){ 
            if (I2C1_write_byte((uint8_t) (data & 0x00FF))) {
              I2C1_stop_bit(); 
              return ( -3 );          
            }
        }else {
            I2C1_stop_bit(); 
            return ( -2 );            
        }            
          
        } else {
          I2C1_stop_bit(); 
          return ( -2 );              
        }
    }

    I2C1_stop_bit();                      

    // test for bus collision
    if (I2C1STATbits.BCL){
        return ( -1 );                
    }
    
    return ( 0 );         
}
 
#ifdef I2C2_AVAILABLE 
int8_t  I2C2_write_word_to_external_device (uint8_t control_byte, uint8_t register_address, uint16_t data){
    
    I2C2_wait_bus_IDLE();                     
    I2C2_start_bit();                     
  
    // test for bus collision
    if (I2C2STATbits.BCL){
        return ( -1 );                
    }
    else {
        // write byte - R/W bit should be 0    
        if ( I2C2_write_byte(control_byte)){
          I2C2_stop_bit(); 
          return (-3);              
        }

        // test for ACK condition received
        if ( !I2C2STATbits.ACKSTAT ){
            
          if (I2C2_write_byte(register_address)) {
            I2C2_stop_bit(); 
            return ( -3 );            
          }
          
        // test for ACK condition received
        if (!I2C2STATbits.ACKSTAT){ 
            if (I2C2_write_byte((uint8_t) (data >> 8))) {
              I2C2_stop_bit(); 
              return ( -3 );          
            }
        }else {
            I2C2_stop_bit(); 
            return ( -2 );            
        }
        
        // test for ACK condition received
        if (!I2C2STATbits.ACKSTAT){ 
            if (I2C2_write_byte((uint8_t) (data & 0x00FF))) {
              I2C2_stop_bit(); 
              return ( -3 );          
            }
        }else {
            I2C2_stop_bit(); 
            return ( -2 );            
        }  
          
          
        } else {
          I2C2_stop_bit(); 
          return ( -2 );              
        }
    }

    I2C2_stop_bit();                      

    // test for bus collision
    if (I2C2STATbits.BCL){
        return ( -1 );                
    }
    
    return ( 0 );         
}
#endif  


//************************************************************
//   I2C1_read_word_from_external_device function implementation
//************************************************************  
int8_t I2C1_read_word_from_external_device (uint8_t control_byte, uint8_t register_address, uint16_t *data){
   
    I2C1_wait_bus_IDLE();                     
    I2C1_start_bit();                     

    // test for bus collision
    if (I2C1STATbits.BCL){
        return (-1);                
    } else {
        if (I2C1_write_byte(control_byte)) {
          I2C1_stop_bit(); 
          return (-3);             
        }
        
        // Check for ACK condition
        if (!I2C1STATbits.ACKSTAT){
          
          // WRITE word address for EEPROM 
          if (I2C1_write_byte(register_address)){
            I2C1_stop_bit(); 
            return (-3);            
          }
        
          // Check for ACK condition
          if (!I2C1STATbits.ACKSTAT) {
              
            I2C1_restart_communication();            
            
            // test for bus collision
            if (I2C1STATbits.BCL){
              return (-1);         
            }

            // write 1 byte - R/W bit should be 1
            if (I2C1_write_byte(control_byte+1)){
              I2C1_stop_bit(); 
              return (-3);         
            }

            // Check for ACK condition
            if (!I2C1STATbits.ACKSTAT){
                // enable master for 1 byte reception  
                I2C1CONbits.RCEN = 1;   
                // check that receive sequence is over
                while (I2C1CONbits.RCEN); 

                // send ACK condition
                I2C1_ACK();    

                // if no error occured the data is valid!
                *data = ((uint16_t) I2C1RCV) << 8;
                
                // enable master for 1 byte reception  
                I2C1CONbits.RCEN = 1;   
                // check that receive sequence is over
                while (I2C1CONbits.RCEN); 

                // send ACK condition
                I2C1_ACK();    

                // if no error occured the data is valid!
                *data |= (uint16_t) I2C1RCV ;                

                // send STOP condition
                I2C1_stop_bit();             

                // test for bus collision
                if (I2C1STATbits.BCL) {
                    return (-1);         
              }
            } else {
              I2C1_stop_bit(); 
              return (-2);          
            }
          } else {
            I2C1_stop_bit(); 
            return (-2);           
          }
        }
        else {
          I2C1_stop_bit(); 
          return (-2);              
        }
    }     
    return (0);  
}
 
#ifdef I2C2_AVAILABLE 
int8_t  I2C2_read_word_from_external_device (uint8_t control_byte, uint8_t register_address, uint16_t *data){
    I2C2_wait_bus_IDLE();                     
    I2C2_start_bit();                     

    // test for bus collision
    if (I2C2STATbits.BCL){
        return (-1);                
    } else {
        if (I2C2_write_byte(control_byte)) {
          I2C2_stop_bit(); 
          return (-3);             
        }
        
        // Check for ACK condition
        if (!I2C2STATbits.ACKSTAT){
          
          // WRITE word address for EEPROM 
          if (I2C2_write_byte(register_address)){
            I2C2_stop_bit(); 
            return (-3);            
          }
        
          // Check for ACK condition
          if (!I2C2STATbits.ACKSTAT) {
              
            I2C2_restart_communication();            
            
            // test for bus collision
            if (I2C2STATbits.BCL){
              return (-1);         
            }

            // write 1 byte - R/W bit should be 1
            if (I2C2_write_byte(control_byte+1)){
              I2C2_stop_bit(); 
              return (-3);         
            }

            // Check for ACK condition
            if (!I2C2STATbits.ACKSTAT){
                // enable master for 1 byte reception  
                I2C2CONbits.RCEN = 1;   
                // check that receive sequence is over
                while (I2C2CONbits.RCEN); 

                // send ACK condition
                I2C2_ACK();    

                // if no error occurred the data is valid!
                *data = ((uint16_t) I2C2RCV) << 8;
                
                // enable master for 1 byte reception  
                I2C2CONbits.RCEN = 1;   
                // check that receive sequence is over
                while (I2C2CONbits.RCEN); 

                // send ACK condition
                I2C2_ACK();    

                // if no error occurred the data is valid!
                *data |= (uint16_t) I2C2RCV ;                

                // send STOP condition
                I2C2_stop_bit();             

                // test for bus collision
                if (I2C2STATbits.BCL) {
                    return (-1);         
              }
            } else {
              I2C2_stop_bit(); 
              return (-2);          
            }
          } else {
            I2C2_stop_bit(); 
            return (-2);           
          }
        }
        else
        {
          I2C2_stop_bit(); 
          return (-2);              
        }
    }     
    return (0);  
}


//************************************************************
//   I2C1_write_array_to_external_device function implementation
//************************************************************  
signed char I2C1_write_array_to_external_device (uint8_t control_byte, uint8_t *data, uint8_t array_size){
    
    uint8_t counter = 0;
    
    I2C1_wait_bus_IDLE();                     
    I2C1_start_bit();                     
  
    // test for bus collision
    if (I2C1STATbits.BCL){
        return ( -1 );                
    }
    else {
        // write byte - R/W bit should be 0    
        if ( I2C1_write_byte(control_byte)){
          I2C1_stop_bit(); 
          return (-3);              
        }

        for (counter = 0; counter < array_size; counter++){
          // test for ACK condition received
            if (!I2C1STATbits.ACKSTAT){ 
                if (I2C1_write_byte(data[counter])) {
                  I2C1_stop_bit(); 
                  return ( -3 );          
                }
            }else {
                I2C1_stop_bit(); 
                return ( -2 );            
            }  
        }                 
    }

    I2C1_stop_bit();                      

    // test for bus collision
    if (I2C1STATbits.BCL){
        return ( -1 );                
    }
    
    return ( 0 );     
}


//************************************************************
//   I2C2_write_array_to_external_device function implementation
//************************************************************  
signed char I2C2_write_array_to_external_device (uint8_t control_byte, uint8_t *data, uint8_t array_size){
    
    uint8_t counter = 0;
    
    I2C2_wait_bus_IDLE();                     
    I2C2_start_bit();                     
  
    // test for bus collision
    if (I2C2STATbits.BCL){
        return ( -1 );                
    }
    else {
        // write byte - R/W bit should be 0    
        if ( I2C2_write_byte(control_byte)){
          I2C2_stop_bit(); 
          return (-3);              
        }

        for (counter = 0; counter < array_size; counter++){
          // test for ACK condition received
            if (!I2C2STATbits.ACKSTAT){ 
                if (I2C2_write_byte(data[counter])) {
                  I2C2_stop_bit(); 
                  return ( -3 );          
                }
            }else {
                I2C2_stop_bit(); 
                return ( -2 );            
            }  
        } 
    }

    I2C2_stop_bit();                      

    // test for bus collision
    if (I2C2STATbits.BCL){
        return ( -1 );                
    }
    
    return ( 0 );     
}


#endif   
