/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 03/08/2019
Last update     : 03/08/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library contains all the functions that are handy for controlling 			
the SPI module
*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "module_SPI.h"


//************************************************************
//               SPI_open  Implementation
//************************************************************
void SPI1_open (uint8_t device_type){
    
    uint16_t data; 
    
	#ifndef MCC_USED
		//Pin mux initialization
		SPI1_SCK_LINE;			         
		SPI1_SDI_LINE;			 
		SPI1_SDO_LINE;		
		SPI1_SS_LINE;	
	#endif
    
    //Disable the SPI module
    SPI1CONbits.ON = 0x00;
    
    if (device_type == SPI_MASTER_DEVICE){
        SPI1CONbits.MSTEN = 0x01;
    } else {
        SPI1CONbits.MSTEN = 0x00;
    }
        
    //Dummy read to clear the buffer
    data = SPI1BUF;
    
    // Clear Interrupts flags
    IFS1bits.SPI1TXIF = 0x00;
    IFS1bits.SPI1RXIF = 0x00;
    
    //Reset errors : Frame, Overflow, Underflow
    SPI1STATbits.FRMERR = 0x00;
    SPI1STATbits.SPIROV = 0x00;
    SPI1STATbits.SPITUR = 0x00;    
    
    //Enable SPI module, ready for RX/TX
    SPI1CONbits.ON = 0x01;
}

#ifdef SPI2_AVAILABLE
    void SPI2_open (uint8_t device_type){
        uint16_t data; 

		#ifndef MCC_USED
			//Pin mux initialization
			SPI2_SCK_LINE;			         
			SPI2_SDI_LINE;			 
			SPI2_SDO_LINE;		
			SPI2_SS_LINE;
		#endif		

        //Disable the SPI module
        SPI2CONbits.ON = 0x00;

        if (device_type == SPI_MASTER_DEVICE){
            SPI2CONbits.MSTEN = 0x01;
        } else {
            SPI2CONbits.MSTEN = 0x00;
        }

        //Dummy read to clear the buffer
        data = SPI2BUF;

        // Clear Interrupts flags
        IFS1bits.SPI2TXIF = 0x00;
        IFS1bits.SPI2RXIF = 0x00;

        //Reset errors : Frame, Overflow, Underflow
        SPI2STATbits.FRMERR = 0x00;
        SPI2STATbits.SPIROV = 0x00;
        SPI2STATbits.SPITUR = 0x00;    

        //Enable SPI module, ready for RX/TX
        SPI2CONbits.ON = 0x01;
    }
#endif


//************************************************************
//               SPI1_close  Implementation
//************************************************************
void SPI1_close (void){
    SPI1CONbits.ON = 0x00;
}

#ifdef SPI2_AVAILABLE
    void SPI2_close (void){
        SPI2CONbits.ON = 0x00;
    }
#endif

//************************************************************
//               SPI1_baudrate  Implementation
//************************************************************
void SPI1_baudrate (uint8_t baudrate){
    SPI1BRG = baudrate;
}

#ifdef SPI2_AVAILABLE
    void SPI2_baudrate (uint8_t baudrate){
        SPI2BRG = baudrate;
    }
#endif

//************************************************************
//               SPI1_mode  Implementation
//************************************************************
void SPI1_mode (uint8_t communication_mode){
    
    switch (communication_mode){
        case SPI_MODE_0: // Data is sent on the rising edge
                        SPI1CONbits.CKE = 0x01;  
                        SPI1CONbits.CKP = 0x00;
                        break;
                        
        case SPI_MODE_1: 
                        SPI1CONbits.CKE = 0x00;  
                        SPI1CONbits.CKP = 0x00;      
                        break;
                        
        case SPI_MODE_2: 
                        SPI1CONbits.CKE = 0x01;  
                        SPI1CONbits.CKP = 0x01;        
                        break;
                        
        case SPI_MODE_3:
                        SPI1CONbits.CKE = 0x00;  
                        SPI1CONbits.CKP = 0x01; 
                        break;                
    }            
}

#ifdef SPI2_AVAILABLE
    void SPI2_mode (uint8_t communication_mode){

    switch (communication_mode){
        case SPI_MODE_0: // Data is sent on the rising edge
                        SPI2CONbits.CKE = 0x01;  
                        SPI2CONbits.CKP = 0x00;
                        break;
                        
        case SPI_MODE_1: 
                        SPI2CONbits.CKE = 0x00;  
                        SPI2CONbits.CKP = 0x00;      
                        break;
                        
        case SPI_MODE_2: 
                        SPI2CONbits.CKE = 0x01;  
                        SPI2CONbits.CKP = 0x01;        
                        break;
                        
        case SPI_MODE_3:
                        SPI2CONbits.CKE = 0x00;  
                        SPI2CONbits.CKP = 0x01; 
                        break;                
        }            
    }
#endif

//************************************************************
//               SPI1_write_byte  Implementation
//************************************************************
int8_t SPI1_write_byte (uint8_t byte_to_send){
  
    uint8_t data;    
            
    //Dummy read to clear the buffer
    data = SPI1BUF;
    
    //Reset errors : Frame, Overflow, Underflow
    //Clear error history
    SPI1STATbits.FRMERR = 0x00;
    SPI1STATbits.SPIROV = 0x00;
    SPI1STATbits.SPITUR = 0x00; 	

    //Reset Interrupt Flag
    IFS1bits.SPI1TXIF = 0x00;
    IFS1bits.SPI1RXIF = 0x00;

    //Send the data out
    SPI1BUF = byte_to_send;  
    
    //Wait the end of transmission
    while(SPI1_is_TX_over()); 

    //No error occured
    return (0);                
}

#ifdef SPI2_AVAILABLE
    int8_t SPI2_write_byte (uint8_t byte_to_send){

    uint8_t data;    
            
    //Dummy read to clear the buffer
    data = SPI2BUF;
    
    //Reset errors : Frame, Overflow, Underflow
    //Clear error history
    SPI2STATbits.FRMERR = 0x00;
    SPI2STATbits.SPIROV = 0x00;
    SPI2STATbits.SPITUR = 0x00; 	

    //Reset Interrupt Flag
    IFS1bits.SPI2TXIF = 0x00;
    IFS1bits.SPI2RXIF = 0x00;

    //Send the data out
    SPI2BUF = byte_to_send;  
    
    //Wait the end of transmission
    while(SPI2_is_TX_over()); 

    //No error occured
    return (0);                 
    }
#endif


//************************************************************
//               SPI1_read_byte  Implementation
//************************************************************
uint8_t SPI1_read_byte (void){
  
    uint8_t data;
    
    data = SPI1BUF;       
  
    //Clear interrupt flag history  
    IFS1bits.SPI1TXIF = 0x00;
    IFS1bits.SPI1RXIF = 0x00;    

    //Write 0xFF to initiate a write
    SPI1BUF = 0xFF;           

    //Wait end of transmission
    while(SPI1_is_TX_over());  
    
    //Return the new byte out of the 0x00 sent out
    return (SPI1BUF);      
}

#ifdef SPI2_AVAILABLE
    uint8_t SPI2_read_byte (void){

        uint8_t data;

        data = SPI2BUF;       

        //Clear interrupt flag history  
        IFS1bits.SPI2TXIF = 0x00;
        IFS1bits.SPI2RXIF = 0x00;    

        //Write 0xFF to initiate a write
        SPI2BUF = 0xFF;           

        //Wait end of transmission
        while(SPI2_is_TX_over());  

        //Return the new byte out of the 0x00 sent out
        return (SPI2BUF);         
    }    
#endif
    
    
    
//************************************************************
//          SPI1_write_read_byte  Implementation
//************************************************************
uint8_t SPI1_write_read_byte (uint8_t write_byte){
  
    uint8_t data;
    
    data = SPI1BUF;       
  
    //Clear interrupt flag history  
    IFS1bits.SPI1TXIF = 0x00;
    IFS1bits.SPI1RXIF = 0x00;    

    //Write 0x00 to initiate a write
    SPI1BUF = write_byte;           

    //Wait end of transmission
    while(SPI1_is_TX_over());  
    
    //Return the new byte out of the byte sent out
    return (SPI1BUF);      
}

#ifdef SPI2_AVAILABLE
    uint8_t SPI2_write_read_byte (uint8_t write_byte){

        uint8_t data;

        data = SPI2BUF;       

        //Clear interrupt flag history  
        IFS1bits.SPI2TXIF = 0x00;
        IFS1bits.SPI2RXIF = 0x00;    

        //Write 0x00 to initiate a write
        SPI2BUF = write_byte;           

        //Wait end of transmission
        while(SPI2_is_TX_over());  

        //Return the new byte out of the byte sent out
        return (SPI2BUF);         
    }    
#endif