/*******************************************************************************
 
Autore : Mauro Laurenti
Versione : 1.0
 
Created on Date : 26/01/2019
Last update     : 26/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "module_UART.h"


//************************************************************
//         UART1_open function implementation
//************************************************************
#ifdef UART1_AVAILABLE 
void UART1_open (uint16_t baud_rate){
    
	#ifndef MCC_USED
		//IO Pin Initialization  (mux)      
		UART1_TX_LINE;
		UART1_RX_LINE;
	#endif
    
    //Module Initialization
    U1STA = 0x00000000;
    U1MODE = 0x00000000;
    
    // 8 bit mode 1 stop bits 
    U1MODEbits.PDSEL0 = 0x00;
    U1MODEbits.STSEL = 0x00;
          
    //Continuos Operation in Idle Mode
    U1MODEbits.SIDL = 0x00;
    
    //Wake the Module on Start bit
    U1MODEbits.SIDL = 0x00;
    
    //TX Idle value 1
    U1STAbits.UTXINV = 0x00;
    
    //Enable RX Buffer
    U1STAbits.URXEN = 0x01;
    
    //Enable TX buffer
    U1STAbits.UTXEN = 0x01;
    
    //Idle State 1
    U1MODEbits.RXINV = 0x00;
    
    
    //Baud rate
            
    switch(baud_rate) {
        case UART_BAUDRATE_9600:
                                U1BRG = UART_BAUDRATE_9600;
                                break;
        case UART_BAUDRATE_19200:
                                U1BRG = UART_BAUDRATE_19200;
                                break;                                
        case UART_BAUDRATE_38400:
                                U1BRG = UART_BAUDRATE_38400;
                                break;
        case UART_BAUDRATE_57600:
                                U1BRG = UART_BAUDRATE_57600;
                                break;
        case UART_BAUDRATE_115200:
                                U1BRG = UART_BAUDRATE_115200;
                                break;
        case UART_BAUDRATE_250000:
                                U1BRG = UART_BAUDRATE_250000;
                                break;
        case UART_BAUDRATE_500000:
                                U1BRG = UART_BAUDRATE_500000;
                                break;
        default:
                                U1BRG = UART_BAUDRATE_9600;
    }
    
    //Turn on the UART1 Module
    U1MODEbits.ON = 0x01;
       
}
#endif

#ifdef UART2_AVAILABLE 

    void UART2_open (uint16_t baud_rate){
    
	#ifndef MCC_USED	
		//IO Pin Initialization  (mux)      
		UART2_TX_LINE;
		UART2_RX_LINE;
	#endif

    //Module Initialization
    U2STA = 0x00000000;
    U2MODE = 0x00000000;
    
    // 8 bit mode 1 stop bits 
    U2MODEbits.PDSEL0 = 0x00;
    U2MODEbits.PDSEL0 = 0x00;
    U2MODEbits.STSEL = 0x00;
      
    
    //Continuos Operation in Idle Mode
    U2MODEbits.SIDL = 0x00;
    
    //Wake the Module on Start bit
    U2MODEbits.SIDL = 0x00;
    
    //TX Idle value 1
    U2STAbits.UTXINV = 0x00;
    
    //Enable RX Buffer
    U2STAbits.URXEN = 0x01;
    
    //Enable TX buffer
    U2STAbits.UTXEN = 0x01;
    
    //Idle State 1
    U2MODEbits.RXINV = 0x00;
    
    
    //Baud rate
            
    switch(baud_rate) {
        case UART_BAUDRATE_9600:
                                U2BRG = UART_BAUDRATE_9600;
                                break;
        case UART_BAUDRATE_19200:
                                U2BRG = UART_BAUDRATE_19200;
                                break;                                
        case UART_BAUDRATE_38400:
                                U2BRG = UART_BAUDRATE_38400;
                                break;
        case UART_BAUDRATE_57600:
                                U2BRG = UART_BAUDRATE_57600;
                                break;
        case UART_BAUDRATE_115200:
                                U2BRG = UART_BAUDRATE_115200;
                                break;
        case UART_BAUDRATE_250000:
                                U2BRG = UART_BAUDRATE_250000;
                                break;
        case UART_BAUDRATE_500000:
                                U2BRG = UART_BAUDRATE_500000;
                                break;
        default:
                                U2BRG = UART_BAUDRATE_9600;
    }
    
    //Turn on the UART2 Module
    U2MODEbits.ON = 0x01;
    }
#endif

//************************************************************
//         UART1_write_byte function implementation
//************************************************************
#ifdef UART1_AVAILABLE 
    void UART1_write_byte (uint8_t data) { 
      U1TXREG = data;
      while (UART1_TX_busy( ));
    }
#endif

#ifdef UART2_AVAILABLE 
    void UART2_write_byte (uint8_t data) { 
      U2TXREG = data;   
      while (UART2_TX_busy( ));
    }
#endif
    
//************************************************************
//         UART1_write_message function implementation
//************************************************************
#ifdef UART1_AVAILABLE 
    void UART1_write_message(const char *data){

        while(*data) {	    
            UART1_write_byte(*data);
            while(UART1_TX_busy());
            data++;              
        }  
    }
#endif

#ifdef UART2_AVAILABLE 
    void UART2_write_message(const char *data){

        while(*data) {	    
            UART2_write_byte(*data);
            while(UART2_TX_busy());
            data++;              
        }  
    }
#endif

//************************************************************
//         UART1_write_string function implementation
//************************************************************
#ifdef UART1_AVAILABLE 
    void UART1_write_string(uint8_t *data){

        while(*data) {	    
            UART1_write_byte(*data);
            while(UART1_TX_busy());
            data++;              
        }  
    }
#endif

#ifdef UART2_AVAILABLE 
    void UART2_write_string(uint8_t *data){

        while(*data) {	    
            UART2_write_byte(*data);
            while(UART2_TX_busy());
            data++;              
        }  
    }
#endif
    
//************************************************************
//         UART1_read_byte function implementation
//************************************************************
#ifdef UART1_AVAILABLE 
    uint8_t UART1_read_byte(void){

      unsigned char data;   

      data = U1RXREG;                      

      return (data);                     
    }
#endif

#ifdef UART2_AVAILABLE 
    uint8_t UART2_read_byte(void){

      unsigned char data;   

      data = U2RXREG;                      

      return (data);                     
    }
#endif